#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the sessions attribute containers."""

import unittest

from plaso.containers import sessions

from tests import test_lib as shared_test_lib


class SessionTest(shared_test_lib.BaseTestCase):
  """Tests for the session attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = sessions.Session()

    expected_attribute_names = [
        'aborted',
        'artifact_filters',
        'command_line_arguments',
        'completion_time',
        'debug_mode',
        'enabled_parser_names',
        'filter_file',
        'identifier',
        'parser_filter_expression',
        'preferred_codepage',
        'preferred_encoding',
        'preferred_language',
        'preferred_time_zone',
        'preferred_year',
        'product_name',
        'product_version',
        'start_time']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)

  def testCopyAttributesFromSessionCompletion(self):
    """Tests the CopyAttributesFromSessionCompletion function."""
    attribute_container = sessions.Session()

    session_completion = sessions.SessionCompletion(
        identifier=attribute_container.identifier)
    attribute_container.CopyAttributesFromSessionCompletion(session_completion)

    with self.assertRaises(ValueError):
      session_completion = sessions.SessionCompletion()
      attribute_container.CopyAttributesFromSessionCompletion(
          session_completion)

  def testCopyAttributesFromSessionConfiguration(self):
    """Tests the CopyAttributesFromSessionConfiguration function."""
    attribute_container = sessions.Session()

    session_configuration = sessions.SessionConfiguration(
        identifier=attribute_container.identifier)
    attribute_container.CopyAttributesFromSessionConfiguration(
        session_configuration)

    with self.assertRaises(ValueError):
      session_configuration = sessions.SessionConfiguration()
      attribute_container.CopyAttributesFromSessionConfiguration(
          session_configuration)

  def testCopyAttributesFromSessionStart(self):
    """Tests the CopyAttributesFromSessionStart function."""
    attribute_container = sessions.Session()

    session_start = sessions.SessionStart(
        identifier=attribute_container.identifier)
    attribute_container.CopyAttributesFromSessionStart(session_start)

  def testCreateSessionCompletion(self):
    """Tests the CreateSessionCompletion function."""
    attribute_container = sessions.Session()

    session_completion = attribute_container.CreateSessionCompletion()
    self.assertIsNotNone(session_completion)
    self.assertEqual(
        session_completion.identifier, attribute_container.identifier)

  def testCreateSessionStart(self):
    """Tests the CreateSessionStart function."""
    attribute_container = sessions.Session()

    session_start = attribute_container.CreateSessionStart()
    self.assertIsNotNone(session_start)
    self.assertEqual(session_start.identifier, attribute_container.identifier)


class SessionCompletionTest(shared_test_lib.BaseTestCase):
  """Tests for the session completion attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = sessions.SessionCompletion()

    expected_attribute_names = [
        'aborted',
        'identifier',
        'timestamp']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


class SessionConfiguration(shared_test_lib.BaseTestCase):
  """Tests for the session configuration attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = sessions.SessionConfiguration()

    expected_attribute_names = [
        'artifact_filters',
        'command_line_arguments',
        'debug_mode',
        'enabled_parser_names',
        'filter_file',
        'identifier',
        'parser_filter_expression',
        'preferred_codepage',
        'preferred_encoding',
        'preferred_language',
        'preferred_time_zone',
        'preferred_year']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


class SessionStartTest(shared_test_lib.BaseTestCase):
  """Tests for the session start attribute container."""

  def testGetAttributeNames(self):
    """Tests the GetAttributeNames function."""
    attribute_container = sessions.SessionStart()

    expected_attribute_names = [
        'identifier',
        'product_name',
        'product_version',
        'timestamp']

    attribute_names = sorted(attribute_container.GetAttributeNames())

    self.assertEqual(attribute_names, expected_attribute_names)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the fake storage writer."""

import unittest

from plaso.containers import events
from plaso.lib import definitions
from plaso.storage.fake import writer as fake_writer

from tests.storage import test_lib
from tests.containers import test_lib as containers_test_lib


class FakeStorageWriterTest(test_lib.StorageTestCase):
  """Tests for the fake storage writer."""

  # pylint: disable=protected-access

  def _AddTestEvents(self, storage_writer):
    """Adds tests events to the storage writer.

    Args:
      storage_writer (FakeStorageWriter): storage writer.

    Returns:
      list[EventObject]: test events.
    """
    test_events = []
    for event, event_data, event_data_stream in (
        containers_test_lib.CreateEventsFromValues(self._TEST_EVENTS)):
      storage_writer.AddAttributeContainer(event_data_stream)

      event_data.SetEventDataStreamIdentifier(event_data_stream.GetIdentifier())
      storage_writer.AddAttributeContainer(event_data)

      event.SetEventDataIdentifier(event_data.GetIdentifier())
      storage_writer.AddAttributeContainer(event)

      test_events.append(event)

    return test_events

  def testAddAttributeContainer(self):
    """Tests the AddAttributeContainer function."""
    event_data_stream = events.EventDataStream()

    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      number_of_containers = storage_writer.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 0)

      storage_writer.AddAttributeContainer(event_data_stream)

      number_of_containers = storage_writer.GetNumberOfAttributeContainers(
          event_data_stream.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

    finally:
      storage_writer.Close()

    with self.assertRaises(IOError):
      storage_writer.AddAttributeContainer(event_data_stream)

  def testAddOrUpdateEventTag(self):
    """Tests the AddOrUpdateEventTag function."""
    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      test_events = self._AddTestEvents(storage_writer)

      event_tag = events.EventTag()
      event_identifier = test_events[1].GetIdentifier()
      event_tag.SetEventIdentifier(event_identifier)

      event_tag.AddLabel('Label1')

      number_of_containers = storage_writer.GetNumberOfAttributeContainers(
          event_tag.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 0)

      storage_writer.AddOrUpdateEventTag(event_tag)

      number_of_containers = storage_writer.GetNumberOfAttributeContainers(
          event_tag.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 1)

      event_tag = events.EventTag()
      event_identifier = test_events[2].GetIdentifier()
      event_tag.SetEventIdentifier(event_identifier)

      event_tag.AddLabel('Label2')

      storage_writer.AddOrUpdateEventTag(event_tag)

      number_of_containers = storage_writer.GetNumberOfAttributeContainers(
          event_tag.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 2)

      event_tag = events.EventTag()
      event_identifier = test_events[1].GetIdentifier()
      event_tag.SetEventIdentifier(event_identifier)

      event_tag.AddLabel('AnotherLabel1')

      storage_writer.AddOrUpdateEventTag(event_tag)

      number_of_containers = storage_writer.GetNumberOfAttributeContainers(
          event_tag.CONTAINER_TYPE)
      self.assertEqual(number_of_containers, 2)

      event_tags = list(storage_writer.GetAttributeContainers(
          event_tag.CONTAINER_TYPE))
      self.assertEqual(event_tags[0].labels, ['Label1', 'AnotherLabel1'])
      self.assertEqual(event_tags[1].labels, ['Label2'])

    finally:
      storage_writer.Close()

  # TODO: add tests for GetFirstWrittenEventSource
  # TODO: add tests for GetNextWrittenEventSource

  def testGetSortedEvents(self):
    """Tests the GetSortedEvents function."""
    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()

    try:
      self._AddTestEvents(storage_writer)

      test_events = list(storage_writer.GetSortedEvents())
      self.assertEqual(len(test_events), 4)

    finally:
      storage_writer.Close()

    # TODO: add test with time range.

  def testOpenClose(self):
    """Tests the Open and Close functions."""
    storage_writer = fake_writer.FakeStorageWriter()
    storage_writer.Open()
    storage_writer.Close()

    storage_writer.Open()
    storage_writer.Close()

    storage_writer = fake_writer.FakeStorageWriter(
        storage_type=definitions.STORAGE_TYPE_TASK)
    storage_writer.Open()
    storage_writer.Close()

    storage_writer.Open()

    with self.assertRaises(IOError):
      storage_writer.Open()

    storage_writer.Close()

    with self.assertRaises(IOError):
      storage_writer.Close()


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Dynamic selected delimiter separated values output module."""

import datetime
import pytz

from dfdatetime import posix_time as dfdatetime_posix_time

from plaso.output import formatting_helper
from plaso.output import manager
from plaso.output import shared_dsv


class DynamicFieldFormattingHelper(formatting_helper.FieldFormattingHelper):
  """Dynamic output module field formatting helper."""

  # TODO: determine why _FormatTimestampDescription is mapped to both
  # timestamp_desc and type.

  # Maps the name of a fields to a a callback function that formats
  # the field value.
  _FIELD_FORMAT_CALLBACKS = {
      'date': '_FormatDate',
      'datetime': '_FormatDateTime',
      'description': '_FormatMessage',
      'description_short': '_FormatMessageShort',
      'display_name': '_FormatDisplayName',
      'filename': '_FormatFilename',
      'host': '_FormatHostname',
      'hostname': '_FormatHostname',
      'inode': '_FormatInode',
      'macb': '_FormatMACB',
      'message': '_FormatMessage',
      'message_short': '_FormatMessageShort',
      'source': '_FormatSourceShort',
      'sourcetype': '_FormatSource',
      'source_long': '_FormatSource',
      'tag': '_FormatTag',
      'time': '_FormatTime',
      'timestamp_desc': '_FormatTimestampDescription',
      'timezone': '_FormatTimeZone',
      'type': '_FormatTimestampDescription',
      'user': '_FormatUsername',
      'username': '_FormatUsername',
      'values': '_FormatValues',
      'yara_match': '_FormatYaraMatch',
      'zone': '_FormatTimeZone'}

  # The field format callback methods require specific arguments hence
  # the check for unused arguments is disabled here.
  # pylint: disable=unused-argument

  def _FormatDate(self, output_mediator, event, event_data, event_data_stream):
    """Formats a date field.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.

    Returns:
      str: date formatted as "YYYY-MM-DD" or "0000-00-00" on error.
    """
    # For now check if event.timestamp is set, to mimic existing behavior of
    # using 0000-00-00 for 0 timestamp values.
    if not event.timestamp:
      return '0000-00-00'

    date_time = event.date_time
    if not date_time or date_time.is_local_time:
      date_time = dfdatetime_posix_time.PosixTimeInMicroseconds(
          timestamp=event.timestamp)

    # Note that GetDateWithTimeOfDay will return the date and time in UTC,
    # so no adjustment for date_time.time_zone_offset is needed.
    year, month, day_of_month, hours, minutes, seconds = (
        date_time.GetDateWithTimeOfDay())

    if output_mediator.time_zone != pytz.UTC:
      try:
        datetime_object = datetime.datetime(
            year, month, day_of_month, hours, minutes, seconds,
            tzinfo=pytz.UTC)

        datetime_object = datetime_object.astimezone(output_mediator.time_zone)

        year = datetime_object.year
        month = datetime_object.month
        day_of_month = datetime_object.day

      except (OSError, OverflowError, TypeError, ValueError):
        year, month, day_of_month = (None, None, None)

    if None in (year, month, day_of_month):
      self._ReportEventError(event, event_data, (
          'unable to copy timestamp: {0!s} to a human readable date. '
          'Defaulting to: "0000-00-00"').format(event.timestamp))
      return '0000-00-00'

    return '{0:04d}-{1:02d}-{2:02d}'.format(year, month, day_of_month)

  def _FormatTimestampDescription(
      self, output_mediator, event, event_data, event_data_stream):
    """Formats a timestamp description field.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.

    Returns:
      str: timestamp description field.
    """
    return event.timestamp_desc or '-'

  def _FormatYaraMatch(
      self, output_mediator, event, event_data, event_data_stream):
    """Formats a Yara match field.

    Args:
      output_mediator (OutputMediator): mediates interactions between output
          modules and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.

    Returns:
      str: Yara match field.
    """
    yara_match = getattr(event_data_stream, 'yara_match', None) or []
    return '; '.join(yara_match) or '-'

  # pylint: enable=unused-argument


class DynamicOutputModule(shared_dsv.DSVOutputModule):
  """Dynamic selected delimiter separated values (DSV) output module."""

  NAME = 'dynamic'
  DESCRIPTION = (
      'Dynamic selection of fields for a separated value output format.')

  SUPPORTS_ADDITIONAL_FIELDS = True
  SUPPORTS_CUSTOM_FIELDS = True

  _DEFAULT_NAMES = [
      'datetime', 'timestamp_desc', 'source', 'source_long', 'message',
      'parser', 'display_name', 'tag']

  _SORT_KEY_FIELD_NAMES = ['datetime', 'display_name', 'message']

  def __init__(self):
    """Initializes an output module."""
    field_formatting_helper = DynamicFieldFormattingHelper()
    super(DynamicOutputModule, self).__init__(
        field_formatting_helper, self._DEFAULT_NAMES)


manager.OutputManager.RegisterOutput(DynamicOutputModule)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Windows Restore Point (rp.log) file parser."""

import unittest

from plaso.parsers import winrestore

from tests.parsers import test_lib


class RestorePointLogParserTest(test_lib.ParserTestCase):
  """Tests for the Windows Restore Point (rp.log) file parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = winrestore.RestorePointLogParser()
    storage_writer = self._ParseFile(['rp.log'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 1)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'creation_time': '2015-03-23T18:38:14.2469544+00:00',
        'data_type': 'windows:restore_point:info',
        'description': 'Software Distribution Service 3.0',
        'restore_point_event_type': 102,
        'restore_point_type': 0}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the default event formatter."""

import unittest

from plaso.formatters import default
from plaso.lib import definitions

from tests.containers import test_lib as containers_test_lib
from tests.formatters import test_lib


class DefaultEventFormatterTest(test_lib.EventFormatterTestCase):
  """Tests for the default event formatter."""

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'description': 'this is beyond words',
       'numeric': 12,
       'text': 'but we\'re still trying to say something about the event',
       'timestamp': 1335791207939596,
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN}]

  def testInitialization(self):
    """Tests the initialization."""
    event_formatter = default.DefaultEventFormatter()
    self.assertIsNotNone(event_formatter)

  def testGetFormatStringAttributeNames(self):
    """Tests the GetFormatStringAttributeNames function."""
    event_formatter = default.DefaultEventFormatter()

    expected_attribute_names = ['attribute_values']

    self._TestGetFormatStringAttributeNames(
        event_formatter, expected_attribute_names)

  def testGetMessage(self):
    """Tests the GetMessage function."""
    event_formatter = default.DefaultEventFormatter()

    _, event_data, _ = containers_test_lib.CreateEventFromValues(
        self._TEST_EVENTS[0])
    event_values = event_data.CopyToDict()

    expected_message = (
        '<WARNING DEFAULT FORMATTER> Attributes: '
        'description: this is beyond words '
        'numeric: 12 '
        'text: but we\'re still trying to say something about the event')

    message = event_formatter.GetMessage(event_values)
    self.assertEqual(message, expected_message)

  def testGetMessageShort(self):
    """Tests the GetMessageShort function."""
    event_formatter = default.DefaultEventFormatter()

    _, event_data, _ = containers_test_lib.CreateEventFromValues(
        self._TEST_EVENTS[0])
    event_values = event_data.CopyToDict()

    expected_message_short = (
        '<DEFAULT> '
        'description: this is beyond words '
        'numeric: 12 '
        'text: but we\'re still...')

    message_short = event_formatter.GetMessageShort(event_values)
    self.assertEqual(message_short, expected_message_short)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Google Chrome history custom event formatter helpers."""

from plaso.formatters import interface
from plaso.formatters import manager


class ChromeHistoryTypedCountFormatterHelper(
    interface.CustomEventFormatterHelper):
  """Google Chrome history typed count formatter helper."""

  IDENTIFIER = 'chrome_history_typed_count'

  def FormatEventValues(self, output_mediator, event_values):
    """Formats event values using the helper.

    Args:
      output_mediator (OutputMediator): output mediator.
      event_values (dict[str, object]): event values.
    """
    typed_count = event_values.get('typed_count', None)
    if typed_count is not None:
      if typed_count == 0:
        url_typed_string = '(URL not typed directly)'
      elif typed_count == 1:
        url_typed_string = '(URL typed 1 time)'
      elif typed_count > 1:
        url_typed_string = '(URL typed {0:d} times)'.format(typed_count)
      else:
        url_typed_string = typed_count

      event_values['url_typed_string'] = url_typed_string


manager.FormattersManager.RegisterEventFormatterHelper(
    ChromeHistoryTypedCountFormatterHelper)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the install history plist plugin."""

import unittest

from plaso.parsers.plist_plugins import install_history

from tests.parsers.plist_plugins import test_lib


class MacOSInstallHistoryPlistPluginTest(test_lib.PlistPluginTestCase):
  """Tests for the install history plist plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plist_name = 'InstallHistory.plist'

    plugin = install_history.MacOSInstallHistoryPlistPlugin()
    storage_writer = self._ParsePlistFileWithPlugin(
        plugin, [plist_name], plist_name)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 7)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'data_type': 'macos:install_history:entry',
        'identifiers': [
            'com.apple.pkg.BaseSystemBinaries',
            'com.apple.pkg.BaseSystemResources',
            'com.apple.pkg.Essentials',
            'com.apple.pkg.BSD',
            'com.apple.pkg.JavaTools',
            'com.apple.pkg.AdditionalEssentials',
            'com.apple.pkg.AdditionalSpeechVoices',
            'com.apple.pkg.AsianLanguagesSupport',
            'com.apple.pkg.MediaFiles',
            'com.apple.pkg.JavaEssentials',
            'com.apple.pkg.OxfordDictionaries',
            'com.apple.pkg.X11redirect',
            'com.apple.pkg.OSInstall',
            'com.apple.pkg.update.compatibility.2013.001'],
        'name': 'OS X',
        'process_name': 'OS X Installer',
        'version': '10.9 (13A603)',
        'written_time': '2013-11-12T02:59:35.000000+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""The status view."""

import ctypes
import re
import sys
import time

try:
  import win32api
  import win32console
except ImportError:
  win32console = None

from dfvfs.lib import definitions as dfvfs_definitions

import plaso

from plaso.cli import tools
from plaso.cli import views
from plaso.lib import definitions


class StatusView(object):
  """Processing status view."""

  MODE_FILE = 'file'
  MODE_LINEAR = 'linear'
  MODE_WINDOW = 'window'

  _SOURCE_TYPES = {
      definitions.SOURCE_TYPE_ARCHIVE: 'archive',
      dfvfs_definitions.SOURCE_TYPE_DIRECTORY: 'directory',
      dfvfs_definitions.SOURCE_TYPE_FILE: 'single file',
      dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_DEVICE: (
          'storage media device'),
      dfvfs_definitions.SOURCE_TYPE_STORAGE_MEDIA_IMAGE: (
          'storage media image')}

  _UNICODE_SURROGATES_RE = re.compile('[\ud800-\udfff]')

  _UNITS_1024 = ['B', 'KiB', 'MiB', 'GiB', 'TiB', 'EiB', 'ZiB', 'YiB']

  _WINAPI_STD_OUTPUT_HANDLE = -11

  _WINAPI_ENABLE_PROCESSED_INPUT = 1
  _WINAPI_ENABLE_LINE_INPUT = 2
  _WINAPI_ENABLE_ECHO_INPUT = 4

  _WINAPI_ANSI_CONSOLE_MODE = (
      _WINAPI_ENABLE_PROCESSED_INPUT | _WINAPI_ENABLE_LINE_INPUT |
      _WINAPI_ENABLE_ECHO_INPUT)

  def __init__(self, output_writer, tool_name):
    """Initializes a status view.

    Args:
      output_writer (OutputWriter): output writer.
      tool_name (str): name of the tool.
    """
    super(StatusView, self).__init__()
    self._artifact_filters = None
    self._filter_file = None
    self._have_ansi_support = not win32console
    self._mode = self.MODE_WINDOW
    self._output_writer = output_writer
    self._source_path = None
    self._source_type = None
    self._status_file = 'status.info'
    self._stdout_output_writer = isinstance(
        output_writer, tools.StdoutOutputWriter)
    self._storage_file_path = None
    self._tool_name = tool_name

    if win32console:
      kernel32 = ctypes.windll.kernel32
      stdout_handle = kernel32.GetStdHandle(self._WINAPI_STD_OUTPUT_HANDLE)
      result = kernel32.SetConsoleMode(
          stdout_handle, self._WINAPI_ANSI_CONSOLE_MODE)
      self._have_ansi_support = result != 0

  def _AddsAnalysisProcessStatusTableRow(self, process_status, table_view):
    """Adds an analysis process status table row.

    Args:
      process_status (ProcessStatus): processing status.
      table_view (CLITabularTableView): table view.
    """
    used_memory = self._FormatSizeInUnitsOf1024(process_status.used_memory)

    events = ''
    if (process_status.number_of_consumed_events is not None and
        process_status.number_of_consumed_events_delta is not None):
      events = '{0:d} ({1:d})'.format(
          process_status.number_of_consumed_events,
          process_status.number_of_consumed_events_delta)

    event_tags = ''
    if (process_status.number_of_produced_event_tags is not None and
        process_status.number_of_produced_event_tags_delta is not None):
      event_tags = '{0:d} ({1:d})'.format(
          process_status.number_of_produced_event_tags,
          process_status.number_of_produced_event_tags_delta)

    reports = ''
    if (process_status.number_of_produced_reports is not None and
        process_status.number_of_produced_reports_delta is not None):
      reports = '{0:d} ({1:d})'.format(
          process_status.number_of_produced_reports,
          process_status.number_of_produced_reports_delta)

    table_view.AddRow([
        process_status.identifier, process_status.pid, process_status.status,
        used_memory, events, event_tags, reports])

  def _AddExtractionProcessStatusTableRow(
      self, process_status, table_view, show_events):
    """Adds an extraction process status table row.

    Args:
      process_status (ProcessStatus): processing status.
      table_view (CLITabularTableView): table view.
      show_events (bool): True if number of events should be shown instead of
          number of event data.
    """
    used_memory = self._FormatSizeInUnitsOf1024(process_status.used_memory)

    sources = ''
    if (process_status.number_of_produced_sources is not None and
        process_status.number_of_produced_sources_delta is not None):
      sources = '{0:d} ({1:d})'.format(
          process_status.number_of_produced_sources,
          process_status.number_of_produced_sources_delta)

    events_or_event_data = ''
    if show_events:
      if (process_status.number_of_produced_events is not None and
          process_status.number_of_produced_events_delta is not None):
        events_or_event_data = '{0:d} ({1:d})'.format(
            process_status.number_of_produced_events,
            process_status.number_of_produced_events_delta)
    else:
      if (process_status.number_of_produced_event_data is not None and
          process_status.number_of_produced_event_data_delta is not None):
        events_or_event_data = '{0:d} ({1:d})'.format(
            process_status.number_of_produced_event_data,
            process_status.number_of_produced_event_data_delta)

    # TODO: shorten display name to fit in 80 chars and show the filename.

    table_view.AddRow([
        process_status.identifier, process_status.pid, process_status.status,
        used_memory, sources, events_or_event_data,
        process_status.display_name])

  def _ClearScreen(self):
    """Clears the terminal/console screen."""
    if self._have_ansi_support:
      # ANSI escape sequence to clear screen.
      self._output_writer.Write('\033[2J')
      # ANSI escape sequence to move cursor to top left.
      self._output_writer.Write('\033[H')

    elif win32console:
      # This version of Windows cmd.exe does not support ANSI escape codes, thus
      # instead we fill the console screen buffer with spaces. The downside of
      # this approach is an annoying flicker.
      top_left_coordinate = win32console.PyCOORDType(0, 0)
      screen_buffer = win32console.GetStdHandle(win32api.STD_OUTPUT_HANDLE)
      screen_buffer_information = screen_buffer.GetConsoleScreenBufferInfo()

      screen_buffer_attributes = screen_buffer_information['Attributes']
      screen_buffer_size = screen_buffer_information['Size']
      console_size = screen_buffer_size.X * screen_buffer_size.Y

      screen_buffer.FillConsoleOutputCharacter(
          ' ', console_size, top_left_coordinate)
      screen_buffer.FillConsoleOutputAttribute(
          screen_buffer_attributes, console_size, top_left_coordinate)
      screen_buffer.SetConsoleCursorPosition(top_left_coordinate)

      # TODO: remove update flicker. For win32console we could set the cursor
      # top left, write the table, clean the remainder of the screen buffer
      # and set the cursor at the end of the table.

  def _FormatSizeInUnitsOf1024(self, size):
    """Represents a number of bytes in units of 1024.

    Args:
      size (int): size in bytes.

    Returns:
      str: human readable string of the size.
    """
    magnitude_1024 = 0
    used_memory_1024 = float(size)
    while used_memory_1024 >= 1024:
      used_memory_1024 /= 1024
      magnitude_1024 += 1

    if 0 < magnitude_1024 <= 7:
      return '{0:.1f} {1:s}'.format(
          used_memory_1024, self._UNITS_1024[magnitude_1024])

    return '{0:d} B'.format(size)

  def _FormatProcessingTime(self, processing_status):
    """Formats the processing time.

    Args:
      processing_status (ProcessingStatus): processing status.

    Returns:
      str: processing time formatted as: "5 days, 12:34:56".
    """
    processing_time = 0
    if processing_status:
      processing_time = time.time() - processing_status.start_time

    processing_time, seconds = divmod(int(processing_time), 60)
    processing_time, minutes = divmod(processing_time, 60)
    days, hours = divmod(processing_time, 24)

    if days == 0:
      days_string = ''
    elif days == 1:
      days_string = '1 day, '
    else:
      days_string = '{0:d} days, '.format(days)

    return '{0:s}{1:02d}:{2:02d}:{3:02d}'.format(
        days_string, hours, minutes, seconds)

  def _PrintAnalysisStatusHeader(self, processing_status):
    """Prints the analysis status header.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    self._output_writer.Write(
        'Storage file\t\t: {0:s}\n'.format(self._storage_file_path))

    processing_time = self._FormatProcessingTime(processing_status)
    self._output_writer.Write(
        'Processing time\t\t: {0:s}\n'.format(processing_time))

    if processing_status and processing_status.events_status:
      self._PrintEventsStatus(processing_status.events_status)

    self._output_writer.Write('\n')

  def _GetPathSpecificationString(self, path_spec):
    """Retrieves a printable string representation of the path specification.

    Args:
      path_spec (dfvfs.PathSpec): path specification.

    Returns:
      str: printable string representation of the path specification.
    """
    path_spec_string = path_spec.comparable

    if self._UNICODE_SURROGATES_RE.search(path_spec_string):
      path_spec_string = path_spec_string.encode(
          'utf-8', errors='surrogateescape')
      path_spec_string = path_spec_string.decode(
          'utf-8', errors='backslashreplace')

    return path_spec_string

  def _PrintAnalysisStatusUpdateFile(self, processing_status):
    """Prints an analysis status update in file mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if processing_status and processing_status.events_status:
      events_status = processing_status.events_status

      with open(self._status_file, 'w', encoding='utf-8') as file_object:
        status_line = (
            'Events: Filtered: {0:d} In time slice: {1:d} Duplicates: {2:d} '
            'MACB grouped: {3:d} Total: {4:d}\n').format(
                events_status.number_of_filtered_events,
                events_status.number_of_events_from_time_slice,
                events_status.number_of_duplicate_events,
                events_status.number_of_macb_grouped_events,
                events_status.total_number_of_events)
        file_object.write(status_line)

  def _PrintAnalysisStatusUpdateLinear(self, processing_status):
    """Prints an analysis status update in linear mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    processing_time = self._FormatProcessingTime(processing_status)
    self._output_writer.Write(
        'Processing time: {0:s}\n'.format(processing_time))

    status_line = (
        '{0:s} (PID: {1:d}) status: {2:s}, events consumed: {3:d}\n').format(
            processing_status.foreman_status.identifier,
            processing_status.foreman_status.pid,
            processing_status.foreman_status.status,
            processing_status.foreman_status.number_of_consumed_events)
    self._output_writer.Write(status_line)

    for worker_status in processing_status.workers_status:
      status_line = (
          '{0:s} (PID: {1:d}) status: {2:s}, events consumed: {3:d}\n').format(
              worker_status.identifier, worker_status.pid, worker_status.status,
              worker_status.number_of_consumed_events)
      self._output_writer.Write(status_line)

    self._output_writer.Write('\n')

  def _PrintAnalysisStatusUpdateWindow(self, processing_status):
    """Prints an analysis status update in window mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if self._stdout_output_writer:
      self._ClearScreen()

    output_text = 'plaso - {0:s} version {1:s}\n\n'.format(
        self._tool_name, plaso.__version__)
    self._output_writer.Write(output_text)

    self._PrintAnalysisStatusHeader(processing_status)

    table_view = views.CLITabularTableView(column_names=[
        'Identifier', 'PID', 'Status', 'Memory', 'Events', 'Tags',
        'Reports'], column_sizes=[23, 7, 15, 15, 15, 15, 0])

    self._AddsAnalysisProcessStatusTableRow(
        processing_status.foreman_status, table_view)

    for worker_status in processing_status.workers_status:
      self._AddsAnalysisProcessStatusTableRow(worker_status, table_view)

    table_view.Write(self._output_writer)
    self._output_writer.Write('\n')

    if processing_status.aborted:
      self._output_writer.Write(
          'Processing aborted - waiting for clean up.\n\n')

    if self._stdout_output_writer:
      # We need to explicitly flush stdout to prevent partial status updates.
      sys.stdout.flush()

  def _PrintExtractionStatusUpdateFile(self, processing_status):
    """Prints an extraction status update in file mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if processing_status and processing_status.tasks_status:
      tasks_status = processing_status.tasks_status

      with open(self._status_file, 'w', encoding='utf-8') as file_object:
        status_line = (
            'Tasks: Queued: {0:d} Processing: {1:d} Merging: {2:d} Abandoned: '
            '{3:d} Total: {4:d}\n').format(
                tasks_status.number_of_queued_tasks,
                tasks_status.number_of_tasks_processing,
                tasks_status.number_of_tasks_pending_merge,
                tasks_status.number_of_abandoned_tasks,
                tasks_status.total_number_of_tasks)
        file_object.write(status_line)

  def _PrintExtractionStatusUpdateLinear(self, processing_status):
    """Prints an extraction status update in linear mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    processing_time = self._FormatProcessingTime(processing_status)
    self._output_writer.Write(
        'Processing time: {0:s}\n'.format(processing_time))

    status_line = (
        '{0:s} (PID: {1:d}) status: {2:s}, event data produced: {3:d}, events '
        'produced: {4:d}, file: {5:s}\n').format(
            processing_status.foreman_status.identifier,
            processing_status.foreman_status.pid,
            processing_status.foreman_status.status,
            processing_status.foreman_status.number_of_produced_event_data,
            processing_status.foreman_status.number_of_produced_events,
            processing_status.foreman_status.display_name)
    self._output_writer.Write(status_line)

    for worker_status in processing_status.workers_status:
      status_line = (
          '{0:s} (PID: {1:d}) status: {2:s}, event data produced: {3:d}, file: '
          '{4:s}\n').format(
              worker_status.identifier, worker_status.pid, worker_status.status,
              worker_status.number_of_produced_events,
              worker_status.display_name)
      self._output_writer.Write(status_line)

    self._output_writer.Write('\n')

  def _PrintExtractionStatusUpdateWindow(self, processing_status):
    """Prints an extraction status update in window mode.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if self._stdout_output_writer:
      self._ClearScreen()

    workers_status = processing_status.workers_status

    show_events = bool(
        processing_status.foreman_status.number_of_produced_events and
        not workers_status)

    output_text = 'plaso - {0:s} version {1:s}\n\n'.format(
        self._tool_name, plaso.__version__)
    self._output_writer.Write(output_text)

    self.PrintExtractionStatusHeader(processing_status)

    if show_events:
      column_name = [
          'Identifier', 'PID', 'Status', 'Memory', 'Sources', 'Events', 'File']
    else:
      column_name = [
          'Identifier', 'PID', 'Status', 'Memory', 'Sources', 'Event Data',
          'File']

    table_view = views.CLITabularTableView(
        column_names=column_name, column_sizes=[15, 7, 15, 15, 15, 15, 0])

    self._AddExtractionProcessStatusTableRow(
        processing_status.foreman_status, table_view, show_events)

    for worker_status in workers_status:
      self._AddExtractionProcessStatusTableRow(
          worker_status, table_view, show_events)

    table_view.Write(self._output_writer)
    self._output_writer.Write('\n')

    if processing_status.aborted:
      self._output_writer.Write(
          'Processing aborted - waiting for clean up.\n\n')

    # TODO: remove update flicker. For win32console we could set the cursor
    # top left, write the table, clean the remainder of the screen buffer
    # and set the cursor at the end of the table.
    if self._stdout_output_writer:
      # We need to explicitly flush stdout to prevent partial status updates.
      sys.stdout.flush()

  def _PrintEventsStatus(self, events_status):
    """Prints the status of the events.

    Args:
      events_status (EventsStatus): events status.
    """
    if events_status:
      table_view = views.CLITabularTableView(
          column_names=['Events:', 'Filtered', 'In time slice', 'Duplicates',
                        'MACB grouped', 'Total'],
          column_sizes=[15, 15, 15, 15, 15, 0])

      table_view.AddRow([
          '', events_status.number_of_filtered_events,
          events_status.number_of_events_from_time_slice,
          events_status.number_of_duplicate_events,
          events_status.number_of_macb_grouped_events,
          events_status.total_number_of_events])

      self._output_writer.Write('\n')
      table_view.Write(self._output_writer)

  def _PrintTasksStatus(self, processing_status):
    """Prints the status of the tasks.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    if processing_status and processing_status.tasks_status:
      tasks_status = processing_status.tasks_status

      table_view = views.CLITabularTableView(
          column_names=['Tasks:', 'Queued', 'Processing', 'Merging',
                        'Abandoned', 'Total'],
          column_sizes=[15, 7, 15, 15, 15, 0])

      table_view.AddRow([
          '', tasks_status.number_of_queued_tasks,
          tasks_status.number_of_tasks_processing,
          tasks_status.number_of_tasks_pending_merge,
          tasks_status.number_of_abandoned_tasks,
          tasks_status.total_number_of_tasks])

      self._output_writer.Write('\n')
      table_view.Write(self._output_writer)

  def GetAnalysisStatusUpdateCallback(self):
    """Retrieves the analysis status update callback function.

    Returns:
      function: status update callback function or None if not available.
    """
    if self._mode == self.MODE_FILE:
      return self._PrintAnalysisStatusUpdateFile

    if self._mode == self.MODE_LINEAR:
      return self._PrintAnalysisStatusUpdateLinear

    if self._mode == self.MODE_WINDOW:
      return self._PrintAnalysisStatusUpdateWindow

    return None

  def GetExtractionStatusUpdateCallback(self):
    """Retrieves the extraction status update callback function.

    Returns:
      function: status update callback function or None if not available.
    """
    if self._mode == self.MODE_FILE:
      return self._PrintExtractionStatusUpdateFile

    if self._mode == self.MODE_LINEAR:
      return self._PrintExtractionStatusUpdateLinear

    if self._mode == self.MODE_WINDOW:
      return self._PrintExtractionStatusUpdateWindow

    return None

  # TODO: refactor to protected method.
  def PrintExtractionStatusHeader(self, processing_status):
    """Prints the extraction status header.

    Args:
      processing_status (ProcessingStatus): processing status.
    """
    self._output_writer.Write(
        'Source path\t\t: {0:s}\n'.format(self._source_path))
    self._output_writer.Write(
        'Source type\t\t: {0:s}\n'.format(self._source_type))

    if self._artifact_filters:
      artifacts_string = ', '.join(self._artifact_filters)
      self._output_writer.Write('Artifact filters\t: {0:s}\n'.format(
          artifacts_string))
    if self._filter_file:
      self._output_writer.Write('Filter file\t\t: {0:s}\n'.format(
          self._filter_file))

    processing_time = self._FormatProcessingTime(processing_status)
    self._output_writer.Write(
        'Processing time\t\t: {0:s}\n'.format(processing_time))

    self._PrintTasksStatus(processing_status)
    self._output_writer.Write('\n')

  def PrintExtractionSummary(
      self, processing_status, number_of_extraction_warnings):
    """Prints a summary of the extraction.

    Args:
      processing_status (ProcessingStatus): processing status.
      number_of_extraction_warnings (int): number of extraction warnings.
    """
    if not processing_status:
      self._output_writer.Write(
          'WARNING: missing processing status information.\n')

    elif not processing_status.aborted:
      if processing_status.error_path_specs:
        self._output_writer.Write('Processing completed with errors.\n')
      else:
        self._output_writer.Write('Processing completed.\n')

      if number_of_extraction_warnings:
        output_text = '\n'.join([
            '',
            ('Number of warnings generated while extracting events: '
             '{0:d}.').format(number_of_extraction_warnings),
            '',
            'Use pinfo to inspect warnings in more detail.',
            ''])
        self._output_writer.Write(output_text)

      if processing_status.error_path_specs:
        output_text = '\n'.join([
            '',
            'Path specifications that could not be processed:',
            ''])
        self._output_writer.Write(output_text)
        for path_spec in processing_status.error_path_specs:
          path_spec_string = self._GetPathSpecificationString(path_spec)
          self._output_writer.Write(path_spec_string)
          self._output_writer.Write('\n')

    self._output_writer.Write('\n')

  def SetMode(self, mode):
    """Sets the mode.

    Args:
      mode (str): status view mode.
    """
    self._mode = mode

  def SetStatusFile(self, path):
    """Sets the status file.

    Args:
      path (str): path of the status file.
    """
    self._status_file = path

  def SetSourceInformation(
      self, source_path, source_type, artifact_filters=None, filter_file=None):
    """Sets the source information.

    Args:
      source_path (str): path of the source.
      source_type (str): source type.
      artifact_filters (Optional[list[str]]): names of artifact definitions to
          use as filters.
      filter_file (Optional[str]): filter file.
    """
    self._artifact_filters = artifact_filters
    self._filter_file = filter_file
    self._source_path = source_path
    self._source_type = self._SOURCE_TYPES.get(source_type, 'UNKNOWN')

  def SetStorageFileInformation(self, storage_file_path):
    """Sets the storage file information.

    Args:
      storage_file_path (str): path to the storage file.
    """
    self._storage_file_path = storage_file_path



#!/usr/bin/env python3
# -*- coding: utf-8 -*- #
"""Tests for the bash history text parser plugin."""

import unittest

from dfvfs.helpers import fake_file_system_builder

from plaso.parsers import text_parser
from plaso.parsers.text_plugins import bash_history

from tests.parsers.text_plugins import test_lib


class BashHistoryTextPluginTest(test_lib.TextPluginTestCase):
  """Testd for the bash history text parser plugin."""

  def _TestEventsFromFile(
      self, storage_writer, expected_number_of_extraction_warnings=0):
    """Validates that all events are as expected.

    Args:
      storage_writer (FakeStorageWriter): storage writer.
      expected_number_of_extraction_warnings (Optional[int]): number of expected
          extraction warnings.
    """
    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, expected_number_of_extraction_warnings)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'command': '/usr/lib/plaso',
        'data_type': 'bash:history:entry',
        'written_time': '2013-10-01T12:36:17+00:00'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)

  def testCheckRequiredFormat(self):
    """Tests for the CheckRequiredFormat method."""
    plugin = bash_history.BashHistoryTextPlugin()

    # Check a bash history file.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'#1380630977\n'
        b'/usr/lib/plaso\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

    # Check a desynchronized bash history file.
    file_system_builder = fake_file_system_builder.FakeFileSystemBuilder()
    file_system_builder.AddFile('/file.txt', (
        b'/sbin/reboot\n'
        b'#1380630977\n'
        b'/usr/lib/plaso\n'))

    file_entry = file_system_builder.file_system.GetFileEntryByPath('/file.txt')

    parser_mediator = self._CreateParserMediator(None, file_entry=file_entry)

    file_object = file_entry.GetFileObject()
    text_reader = text_parser.EncodedTextReader(file_object)
    text_reader.ReadLines()

    result = plugin.CheckRequiredFormat(parser_mediator, text_reader)
    self.assertTrue(result)

  def testProcess(self):
    """Tests the Process function."""
    plugin = bash_history.BashHistoryTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(['bash_history'], plugin)

    self._TestEventsFromFile(storage_writer)

  def testProcessWithDesynchronizedFile(self):
    """Tests the Process function with a desynchronized file.

    A desynchronized bash history file will start with the command line
    instead of the timestamp.
    """
    plugin = bash_history.BashHistoryTextPlugin()
    storage_writer = self._ParseTextFileWithPlugin(
        ['bash_history_desync'], plugin)

    self._TestEventsFromFile(
        storage_writer, expected_number_of_extraction_warnings=1)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Text parser plugin for Snort3/Suricata fast-log alert log files.

Snort3/Suricata fast.log format is a single line log format as shown below.
The following variants are known:

  Snort3:
    Month/Day-Hour:Minute:Second:FractionOfSeconds [**] [sid] "string" [**] \\
    [Classification: string] [Priority : int] {protocol} \\
    SOURCE_IP:SOURCE_PORT -> DESTINATION_IP:DESTINATION_PORT

  Suricata:
    Year/Month/Day-Hour:Minute:Second:FractionOfSeconds [**] [sid] string \\
    [**] [Classification: string] [Priority : int] {protocol} \\
    SOURCE_IP:SOURCE_PORT -> DESTINATION_IP:DESTINATION_PORT

Also see:
  https://suricata.readthedocs.io/en/suricata-6.0.0/configuration/suricata-yaml.html#line-based-alerts-log-fast-log
"""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.lib import yearless_helper
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class SnortFastAlertEventData(events.EventData):
  """Snort3/Suricata fast-log alert event data.

  Attributes:
    classification (str): classification of the alert.
    destination_ip (str): destination IP-address.
    destination_port (int): destination TCP/UDP port number.
    last_written_time (dfdatetime.DateTimeValues): entry last written date and
        time.
    message (str): message associated with the alert.
    priority (int): priorty, ranging from 1 (high) to 4 (very low).
    rule_identifier (str): identifier of the Snort3/Suricata rule that generated
        the alert.
    source_ip (str): source IP-address.
    source_port (int): source TCP/UDP port number.
  """

  DATA_TYPE = 'snort:fastlog:alert'

  def __init__(self):
    """Initializes event data."""
    super(SnortFastAlertEventData, self).__init__(data_type=self.DATA_TYPE)
    self.classification = None
    self.destination_ip = None
    self.destination_port = None
    self.last_written_time = None
    self.message = None
    self.priority = None
    self.protocol = None
    self.rule_identifier = None
    self.source_ip = None
    self.source_port = None


class SnortFastLogTextPlugin(
    interface.TextPlugin, yearless_helper.YearLessLogFormatHelper):
  """Text parser plugin for Snort3/Suricata fast-log alert log files."""

  NAME = 'snort_fastlog'
  DATA_FORMAT = 'Snort3/Suricata fast-log alert log (fast.log) file'

  _INTEGER = pyparsing.Word(pyparsing.nums).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _SIX_DIGITS = pyparsing.Word(pyparsing.nums, exact=6).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _DATE_MONTH_DAY = (
      _TWO_DIGITS + pyparsing.Suppress('/') + _TWO_DIGITS)

  _DATE_YEAR_MONTH_DAY = (
      _TWO_DIGITS + pyparsing.Suppress('/') + _DATE_MONTH_DAY)

  # Date and time values are formatted as: MM/DD-hh:mm:ss.###### or
  # YY/MM/DD-hh:mm:ss.######
  # For example: 12/28-12:55:38.765402 or 10/05/10-10:08:59.667372

  _DATE_TIME = (
      (_DATE_YEAR_MONTH_DAY | _DATE_MONTH_DAY) + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress('.') + _SIX_DIGITS)

  _IP_ADDRESS = (
      pyparsing.pyparsing_common.ipv4_address |
      pyparsing.pyparsing_common.ipv6_address)

  _MESSAGE = (
      pyparsing.Optional(pyparsing.Suppress('"')) +
      pyparsing.Combine(pyparsing.OneOrMore(
          pyparsing.Word(pyparsing.printables, excludeChars='["') |
          pyparsing.White(' ', max=2))).setResultsName('message') +
      pyparsing.Optional(pyparsing.Suppress('"')))

  _RULE = (
      pyparsing.Suppress('[') +
      pyparsing.Combine(
          _INTEGER + pyparsing.Literal(':') +
          _INTEGER + pyparsing.Literal(':') +
          _INTEGER).setResultsName('rule_identifier') +
      pyparsing.Suppress(']'))

  _CLASSIFICATION = (
      pyparsing.Suppress('[Classification:') +
      pyparsing.Regex('[^]]*').setResultsName('classification') +
      pyparsing.Suppress(']'))

  _PRIORITY = (
      pyparsing.Suppress('[Priority:') +
      _INTEGER.setResultsName('priority') +
      pyparsing.Suppress(']'))

  _SOURCE_IP_ADDRESS_AND_PORT = (
      _IP_ADDRESS.setResultsName('source_ip_address') +
      pyparsing.Suppress(':') +
      _INTEGER.setResultsName('source_port'))

  _DESTINATION_IP_ADDRESS_AND_PORT = (
      _IP_ADDRESS.setResultsName('destination_ip_address') +
      pyparsing.Suppress(':') +
      _INTEGER.setResultsName('destination_port'))

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  _LOG_LINE = (
      _DATE_TIME.setResultsName('date_time') +
      pyparsing.Suppress('[**]') +
      _RULE +
      _MESSAGE +
      pyparsing.Suppress('[**]') +
      pyparsing.Optional(_CLASSIFICATION) +
      pyparsing.Optional(_PRIORITY) +
      pyparsing.Suppress('{') +
      pyparsing.Word(pyparsing.alphanums).setResultsName('protocol') +
      pyparsing.Suppress('}') + (
          _IP_ADDRESS.setResultsName('source_ip_address') ^
          _SOURCE_IP_ADDRESS_AND_PORT) +
      pyparsing.Suppress('->') + (
          _IP_ADDRESS.setResultsName('destination_ip_address') ^
          _DESTINATION_IP_ADDRESS_AND_PORT) +
      _END_OF_LINE)

  _LINE_STRUCTURES = [('log_line', _LOG_LINE)]

  VERIFICATION_GRAMMAR = _LOG_LINE

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    event_data = SnortFastAlertEventData()
    event_data.classification = self._GetValueFromStructure(
        structure, 'classification')
    event_data.destination_ip = self._GetValueFromStructure(
        structure, 'destination_ip_address')
    event_data.destination_port = self._GetValueFromStructure(
        structure, 'destination_port')
    event_data.last_written_time = self._ParseTimeElements(
        time_elements_structure)
    event_data.message = self._GetStringValueFromStructure(
        structure, 'message')
    event_data.priority = self._GetValueFromStructure(structure, 'priority')
    event_data.protocol = self._GetValueFromStructure(structure, 'protocol')
    event_data.source_ip = self._GetValueFromStructure(
        structure, 'source_ip_address')
    event_data.source_port = self._GetValueFromStructure(
        structure, 'source_port')
    event_data.rule_identifier = self._GetValueFromStructure(
        structure, 'rule_identifier')

    parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      has_year = len(time_elements_structure) == 7

      if has_year:
        year, month, day_of_month, hours, minutes, seconds, microseconds = (
            time_elements_structure)

        year += 2000
      else:
        month, day_of_month, hours, minutes, seconds, microseconds = (
            time_elements_structure)

        self._UpdateYear(month)

        year = self._GetRelativeYear()

      time_elements_tuple = (
          year, month, day_of_month, hours, minutes, seconds, microseconds)

      date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
          is_delta=(not has_year), time_elements_tuple=time_elements_tuple)

      date_time.is_local_time = True

      return date_time

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    self._SetEstimatedYear(parser_mediator)

    time_elements_structure = self._GetValueFromStructure(
        structure, 'date_time')

    try:
      self._ParseTimeElements(time_elements_structure)
    except errors.ParseError:
      return False

    return True


text_parser.TextLogParser.RegisterPlugin(SnortFastLogTextPlugin)



# -*- coding: utf-8 -*-
"""JSON-L parser plugin for Azure application gateway access log files."""

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.parsers import jsonl_parser
from plaso.parsers.jsonl_plugins import interface


class AzureApplicationGatewayAccessEventData(events.EventData):
  """Azure application gateway access log event data.

  Attributes:
    client_ip (str): Client IP address of the request.
    client_port (int): Client TCP/UDP port for the request.
    client_response_time (int): Duration, in seconds, from the first byte of
        a client request to be processed up to the first byte sent as response
        to the client.
    host (str): Address listed in the host header of the request. If rewritten
        using header rewrite, contains the updated host name.
    http_method (str): HTTP method used by the request.
    http_status (int): HTTP status code returned to the client from application
        gateway.
    http_version (str): HTTP version of the request.
    instance_identifier (str): Application gateway instance that served
        the request.
    original_host (str): Original request host name.
    original_request_uri (str): Original request URL, including arguments.
    received_bytes (int): Size of packet received, in bytes.
    recorded_time (dfdatetime.DateTimeValues): date and time the log entry
        was recorded.
    request_query (str): Server-Routed: Back-end pool instance that was sent
        the request. X-AzureApplicationGateway-LOG-ID: Correlation ID used for
        the request. It can be used to troubleshoot traffic issues on
        the back-end servers. SERVER-STATUS: HTTP response code that application
        gateway received from the back-end.
    request_uri (str): URI of the received request.
    sent_bytes (int): Size of packet sent, in bytes.
    server_response_latency (str): Latency of the response (in seconds) from
        the back-end server.
    server_routed (str): The back-end server that application gateway routes
        the request to.
    server_status (str): HTTP status code of the back-end server.
    ssl_cipher (str): Cipher suite being used for TLS communication.
    ssl_client_certificate_fingerprint (str): Fingerprint of the SSL client
        certificate.
    ssl_client_certificate_issuer_name (str): Name of the issuer of the SSL
        client certificate.
    ssl_client_verify (str): TODO.
    ssl_enabled (str): Whether communication to the back-end pools used TLS.
        Valid values are on and off.
    ssl_protocol (str): The SSL/TLS protocol used.
    time_taken (double): Duration, in seconds, that it takes for the first byte
        of a client request to be processed and its last-byte sent in
        the response to the client. It's important to note that the Time-Taken
        field usually includes the time that the request and response packets
        are traveling over the network.
    transaction_id (str): Unique identifier to correlate the request received
        from the client
    user_agent (str): User agent from the HTTP request header.
    waf_evaluation_time (str): Duration, in seconds, that it takes for
        the request to be processed by the WAF.
    waf_mode (str): Value can be either Detection or Prevention.
  """

  DATA_TYPE = 'azure:application_gateway_access:entry'

  def __init__(self):
    """Initializes event data."""
    super(AzureApplicationGatewayAccessEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.client_ip = None
    self.client_port = None
    self.client_response_time = None
    self.host = None
    self.http_method = None
    self.http_status = None
    self.http_version = None
    self.instance_identifier = None
    self.original_host = None
    self.original_request_uri = None
    self.received_bytes = None
    self.recorded_time = None
    self.request_query = None
    self.request_uri = None
    self.sent_bytes = None
    self.server_response_latency = None
    self.server_routed = None
    self.server_status = None
    self.ssl_cipher = None
    self.ssl_client_certificate_fingerprint = None
    self.ssl_client_certificate_issuer_name = None
    self.ssl_client_verify = None
    self.ssl_enabled = None
    self.ssl_protocol = None
    self.time_taken = None
    self.transaction_identifier = None
    self.user_agent = None
    self.waf_evaluation_time = None
    self.waf_mode = None


class AzureApplicationGatewayAccessLogJSONLPlugin(interface.JSONLPlugin):
  """JSON-L parser plugin for Azure application gateway access log files."""

  NAME = 'azure_application_gateway_access_log'
  DATA_FORMAT = 'Azure Application Gateway access log'

  def _ParseRecord(self, parser_mediator, json_dict):
    """Parses an Azure application gateway access log record.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      json_dict (dict): JSON dictionary of the log record.
    """
    properties_json_dict = self._GetJSONValue(
        json_dict, 'properties', default_value={})

    event_data = AzureApplicationGatewayAccessEventData()
    event_data.client_ip = self._GetJSONValue(
        properties_json_dict, 'clientIP')
    event_data.client_port = self._GetJSONValue(
        properties_json_dict, 'clientPort')
    event_data.client_response_time = self._GetJSONValue(
        properties_json_dict, 'clientResponseTime')
    event_data.host = self._GetJSONValue(
        properties_json_dict, 'host')
    event_data.http_method = self._GetJSONValue(
        properties_json_dict, 'httpMethod')
    event_data.http_status = self._GetJSONValue(
        properties_json_dict, 'httpStatus')
    event_data.http_version = self._GetJSONValue(
        properties_json_dict, 'httpVersion')
    event_data.instance_identifier = self._GetJSONValue(
        properties_json_dict, 'instanceId')
    event_data.original_host = self._GetJSONValue(
        properties_json_dict, 'originalHost')
    event_data.original_request_uri = self._GetJSONValue(
        properties_json_dict, 'originalRequestUriWithArgs')
    event_data.received_bytes = self._GetJSONValue(
        properties_json_dict, 'receivedBytes')
    event_data.recorded_time = self._ParseISO8601DateTimeString(
        parser_mediator, json_dict, 'timeStamp')
    event_data.request_query = self._GetJSONValue(
        properties_json_dict, 'requestQuery')
    event_data.request_uri = self._GetJSONValue(
        properties_json_dict, 'requestUri')
    event_data.sent_bytes = self._GetJSONValue(
        properties_json_dict, 'sentBytes')
    event_data.server_response_latency = self._GetJSONValue(
        properties_json_dict, 'serverResponseLatency')
    event_data.server_routed = self._GetJSONValue(
        properties_json_dict, 'serverRouted')
    event_data.server_status = self._GetJSONValue(
        properties_json_dict, 'serverStatus')
    event_data.ssl_cipher = self._GetJSONValue(
        properties_json_dict, 'sslCipher')
    event_data.ssl_client_certificate_fingerprint = self._GetJSONValue(
        properties_json_dict, 'sslClientCertificateFingerprint')
    event_data.ssl_client_certificate_issuer_name = self._GetJSONValue(
        properties_json_dict, 'sslClientCertificateIssuerName')
    event_data.ssl_client_verify = self._GetJSONValue(
        properties_json_dict, 'sslClientVerify')
    event_data.ssl_enabled = self._GetJSONValue(
        properties_json_dict, 'sslEnabled')
    event_data.ssl_protocol = self._GetJSONValue(
        properties_json_dict, 'sslProtocol')
    event_data.time_taken = self._GetJSONValue(
        properties_json_dict, 'timeTaken')
    event_data.transaction_identifier = self._GetJSONValue(
        properties_json_dict, 'transactionId')
    event_data.user_agent = self._GetJSONValue(
        properties_json_dict, 'userAgent')
    event_data.waf_evaluation_time = self._GetJSONValue(
        properties_json_dict, 'WAFEvaluationTime')
    event_data.waf_mode = self._GetJSONValue(
        properties_json_dict, 'WAFMode')

    parser_mediator.ProduceEventData(event_data)

  def CheckRequiredFormat(self, json_dict):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      json_dict (dict): JSON dictionary of the log record.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    operation_name = self._GetJSONValue(json_dict, 'operationName')
    properties = self._GetJSONValue(json_dict, 'properties')
    iso8601_string = self._GetJSONValue(json_dict, 'timeStamp')

    if (None in (operation_name, properties, iso8601_string) or
        operation_name != 'ApplicationGatewayAccess'):
      return False

    date_time = dfdatetime_time_elements.TimeElementsInMicroseconds()

    try:
      date_time.CopyFromStringISO8601(iso8601_string)
    except ValueError:
      return False

    return True


jsonl_parser.JSONLParser.RegisterPlugin(
    AzureApplicationGatewayAccessLogJSONLPlugin)



# -*- coding: utf-8 -*-
"""Text parser plugin for AWS ELB access logs.

The AWS documentation is not clear about the meaning of the "target_port_list"
field. The assumption is that it refers to a list of possible backend instances'
IP addresses that could receive the client's request. This parser stores the
"target_port_list" data in the "destination_list" attribute of an EventData
object.

Also see:
  https://docs.aws.amazon.com/elasticloadbalancing/latest/application/load-balancer-access-logs.html
  https://docs.aws.amazon.com/elasticloadbalancing/latest/classic/access-log-collection.html
  https://docs.aws.amazon.com/elasticloadbalancing/latest/network/load-balancer-access-logs.html
"""

import pyparsing

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import text_parser
from plaso.parsers.text_plugins import interface


class AWSELBEventData(events.EventData):
  """AWS Elastic Load Balancer access log event data.

  Attributes:
    actions_executed (str): The actions taken when processing the request.
    alpn_back_end_protocol (str): The application protocol negotiated with
        the target, in string format. If no ALPN policy is configured in
        the TLS listener, no matching protocol is found, or no valid protocol
        list is sent, this value is set to -.
        (only for network load balancer logs)
    alpn_client_preference_list (str): The value of the
        application_layer_protocol_negotiation extension in the client hello
        message. This value is URL-encoded. Each protocol is enclosed in double
        quotes and protocols are separated by a comma. If no ALPN policy is
        configured in the TLS listener, no valid client hello message is sent,
        or the extension is not present, this value is set to -. The string is
        truncated if it is longer than 256 bytes.
        (only for network load balancer logs)
    alpn_front_end_protocol (str): The application protocol negotiated with
        the client, in string format. If no ALPN policy is configured in
        the TLS listener, no matching protocol is found, or no valid protocol
        list is sent, this value is set to -.
        (only for network load balancer logs)
    chosen_cert_arn (str): The ARN of the certificate
        presented to the source.
    chosen_cert_serial (str): Reserved for future use.
        This value is always set to -.
        (only for network load balancer logs)
    classification (str): The classification for desync mitigation.
    classification_reason (str): The classification reason code.
    connection_duration (str): duration of the connection to complete, from
        start to closure, in milliseconds.
        (only for network load balancer logs)
    destination_group_arn (str): The Amazon Resource Name (ARN) of the
        destination group.
    destination_ip_address (str): The IP address of the destination
        that processed this request.
    destination_list (str): A space-delimited list of IP addresses
        and ports for the destinations that processed this request.
    destination_port (int): The port of the destination that processed
        this request.
    destination_processing_duration (str): duration from the time the load
        balancer sent the request to a destination until the destination
        started to send the response headers.
    destination_status_code (int): The status code of the response
        from the destination.
    destination_status_code_list (str): A space-delimited list of status codes.
    domain_name (str): The SNI domain provided by the
        source during the TLS handshake.
    error_reason (str): The error reason code, enclosed in
        double quotes.
    handshake_duration (str): duration of the handshake to complete after
        the TCP connection is established, including client-side delays,
        in milliseconds. This time is included in the connection_duration field.
        (only for network load balancer logs)
    incoming_tls_alert (str): The integer value of TLS alerts received by the
        load balancer from the client, if present.
        (only for network load balancer logs)
    listener (str): The resource ID of the TLS listener for the connection.
        (only for network load balancer logs)
    matched_rule_priority (int): The priority value of the rule that
        matched the request.
    received_bytes (int): The size of the request, in bytes, received from
        the source.
    redirect_url (str): The URL of the redirect destination.
    request_processing_duration (str): total duration from the time the load
        balancer received the request until the time it sent the request to
        a destination.
    request_time (dfdatetime.DateTimeValues): date and time a request
        was sent.
    request_type (str): The type of request or connection.
    resource_identifier (str): The resource ID of the load balancer.
    response_processing_duration (str): duration of processing a response.
    response_time (dfdatetime.DateTimeValues): date and time a response
        was sent.
    sent_bytes (int): The size of the response, in bytes, sent to the source.
    ssl_cipher (str): The SSL cipher of the HTTPS listener.
    ssl_protocol (str): The SSL protocol of the HTTPS listener.
    source_ip_address (str): The IP address of the requesting source.
    source_port (int): The port of the requesting source.
    tls_cipher (str): The cipher suite negotiated with the client, in OpenSSL
        format. If TLS negotiation does not complete, this value is set to -.
        (only for network load balancer logs)
    tls_named_group (str): Reserved for future use.
        This value is always set to -.
        (only for network load balancer logs)
    tls_protocol_version (str): The TLS protocol negotiated with the client,
        in string format. If TLS negotiation does not complete,
        this value is set to -.
        (only for network load balancer logs)
    trace_identifier (str): The contents of the X-Amzn-Trace-Id header.
    user_agent (str): A User-Agent string.
    version (str): The version of the log entry.
        (only for network load balancer logs)
  """

  DATA_TYPE = 'aws:elb:access'

  def __init__(self):
    """Initializes event data."""
    super(AWSELBEventData, self).__init__(data_type=self.DATA_TYPE)
    self.actions_executed = None
    self.alpn_back_end_protocol = None
    self.alpn_client_preference_list = None
    self.alpn_front_end_protocol = None
    self.chosen_cert_arn = None
    self.chosen_cert_serial = None
    self.classification = None
    self.classification_reason = None
    self.connection_duration = None
    self.destination_group_arn = None
    self.destination_ip_address = None
    self.destination_list = None
    self.destination_port = None
    self.destination_processing_duration = None
    self.destination_status_code = None
    self.destination_status_code_list = None
    self.domain_name = None
    self.elb_status_code = None
    self.error_reason = None
    self.handshake_duration = None
    self.incoming_tls_alert = None
    self.listener = None
    self.matched_rule_priority = None
    self.received_bytes = None
    self.redirect_url = None
    self.request = None
    self.request_processing_duration = None
    self.request_time = None
    self.request_type = None
    self.resource_identifier = None
    self.response_processing_duration = None
    self.response_time = None
    self.sent_bytes = None
    self.source_ip_address = None
    self.source_port = None
    self.ssl_cipher = None
    self.ssl_protocol = None
    self.tls_cipher = None
    self.tls_named_group = None
    self.tls_protocol_version = None
    self.trace_identifier = None
    self.user_agent = None
    self.version = None


class AWSELBTextPlugin(interface.TextPlugin):
  """Text parser plugin for AWS ELB access log files."""

  NAME = 'aws_elb_access'
  DATA_FORMAT = 'AWS ELB Access log file'

  ENCODING = 'utf-8'

  _TWO_DIGITS = pyparsing.Word(pyparsing.nums, exact=2).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _FOUR_DIGITS = pyparsing.Word(pyparsing.nums, exact=4).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _SIX_DIGITS = pyparsing.Word(pyparsing.nums, exact=6).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _BLANK = pyparsing.Literal('"-"') | pyparsing.Literal('-')

  _WORD = pyparsing.Word(pyparsing.printables) | _BLANK

  _INTEGER = pyparsing.Word(pyparsing.nums).setParseAction(
      lambda tokens: int(tokens[0], 10))

  _UNSIGNED_INTEGER = _INTEGER | _BLANK

  _SIGNED_INTEGER = pyparsing.Word('-', pyparsing.nums) | _UNSIGNED_INTEGER

  _FLOATING_POINT = (
      pyparsing.Word(pyparsing.nums + '.') | pyparsing.Literal('-1'))

  _IP_ADDRESS = (
      pyparsing.pyparsing_common.ipv4_address |
      pyparsing.pyparsing_common.ipv6_address)

  _PORT = pyparsing.Word(pyparsing.nums, max=6).setParseAction(
      lambda tokens: int(tokens[0], 10)) | _BLANK

  _SOURCE_IP_ADDRESS_AND_PORT = pyparsing.Group(
      _IP_ADDRESS.setResultsName('source_ip_address') +
      pyparsing.Suppress(':') + _PORT.setResultsName('source_port') | _BLANK)

  _DESTINATION_IP_ADDRESS_AND_PORT = pyparsing.Group(
      _IP_ADDRESS.setResultsName('destination_ip_address') +
      pyparsing.Suppress(':') + _PORT.setResultsName('destination_port') |
      _BLANK)

  _DATE_TIME_ISOFORMAT_STRING = (
      _FOUR_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('T') +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress('.') +
      _SIX_DIGITS + pyparsing.Suppress('Z'))

  _DATE_TIME_ISOFORMAT_STRING_WITHOUT_TIMEZONE = (
      _FOUR_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('-') +
      _TWO_DIGITS + pyparsing.Suppress('T') +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS + pyparsing.Suppress(':') +
      _TWO_DIGITS)

  _REQUEST = pyparsing.quotedString.setResultsName(
      'request').setParseAction(pyparsing.removeQuotes)

  _USER_AGENT = pyparsing.quotedString.setResultsName(
      'user_agent').setParseAction(pyparsing.removeQuotes)

  _ALPN_CLIENT_PREFERENCE_LIST = pyparsing.quotedString.setResultsName(
      'alpn_client_preference_list').setParseAction(pyparsing.removeQuotes)

  _END_OF_LINE = pyparsing.Suppress(pyparsing.LineEnd())

  # A log line is defined as in the AWS ELB documentation
  _APPLICATION_LOG_LINE = (
      _WORD.setResultsName('request_type') +
      _DATE_TIME_ISOFORMAT_STRING.setResultsName('response_time') +
      _WORD.setResultsName('resource_identifier') +
      _SOURCE_IP_ADDRESS_AND_PORT.setResultsName('source_ip_port') +
      _DESTINATION_IP_ADDRESS_AND_PORT.setResultsName('destination_ip_port') +
      _FLOATING_POINT.setResultsName('request_processing_duration') +
      _FLOATING_POINT.setResultsName('destination_processing_duration') +
      _FLOATING_POINT.setResultsName('response_processing_duration') +
      _UNSIGNED_INTEGER.setResultsName('elb_status_code') +
      _UNSIGNED_INTEGER.setResultsName('destination_status_code') +
      _UNSIGNED_INTEGER.setResultsName('received_bytes') +
      _UNSIGNED_INTEGER.setResultsName('sent_bytes') +
      _REQUEST +
      _USER_AGENT +
      _WORD.setResultsName('ssl_cipher') +
      _WORD.setResultsName('ssl_protocol') +
      _WORD.setResultsName('destination_group_arn') +
      pyparsing.quotedString.setResultsName(
          'trace_identifier').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'domain_name').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'chosen_cert_arn').setParseAction(pyparsing.removeQuotes) +
      _SIGNED_INTEGER.setResultsName('matched_rule_priority') +
      _DATE_TIME_ISOFORMAT_STRING.setResultsName('request_time') +
      pyparsing.quotedString.setResultsName(
          'actions_executed').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'redirect_url').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'error_reason').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'destination_list').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'destination_status_code_list').setParseAction(
              pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'classification').setParseAction(pyparsing.removeQuotes) +
      pyparsing.quotedString.setResultsName(
          'classification_reason').setParseAction(pyparsing.removeQuotes) +
      _END_OF_LINE)

  _NETWORK_LOG_LINE = (
      _WORD.setResultsName('request_type') +
      _WORD.setResultsName('version') +
      _DATE_TIME_ISOFORMAT_STRING_WITHOUT_TIMEZONE.setResultsName(
          'response_time') +
      _WORD.setResultsName('resource_identifier') +
      _WORD.setResultsName('listener') +
      _SOURCE_IP_ADDRESS_AND_PORT.setResultsName('source_ip_port') +
      _DESTINATION_IP_ADDRESS_AND_PORT.setResultsName('destination_ip_port') +
      _UNSIGNED_INTEGER.setResultsName('connection_duration') +
      _UNSIGNED_INTEGER.setResultsName('handshake_duration') +
      _UNSIGNED_INTEGER.setResultsName('received_bytes') +
      _UNSIGNED_INTEGER.setResultsName('sent_bytes') +
      _WORD.setResultsName('incoming_tls_alert') +
      _WORD.setResultsName('chosen_cert_arn') +
      _WORD.setResultsName('chosen_cert_serial') +
      _WORD.setResultsName('tls_cipher') +
      _WORD.setResultsName('tls_protocol_version') +
      _WORD.setResultsName('tls_named_group') +
      _WORD.setResultsName('domain_name') +
      _WORD.setResultsName('alpn_front_end_protocol') +
      _WORD.setResultsName('alpn_back_end_protocol') +
      (_ALPN_CLIENT_PREFERENCE_LIST | pyparsing.Literal('-')) +
      _END_OF_LINE)

  _CLASSIC_LOG_LINE = (
      _DATE_TIME_ISOFORMAT_STRING.setResultsName('response_time') +
      _WORD.setResultsName('resource_identifier') +
      _SOURCE_IP_ADDRESS_AND_PORT.setResultsName('source_ip_port') +
      _DESTINATION_IP_ADDRESS_AND_PORT.setResultsName('destination_ip_port') +
      _FLOATING_POINT.setResultsName('request_processing_duration') +
      _FLOATING_POINT.setResultsName('destination_processing_duration') +
      _FLOATING_POINT.setResultsName('response_processing_duration') +
      _UNSIGNED_INTEGER.setResultsName('elb_status_code') +
      _UNSIGNED_INTEGER.setResultsName('destination_status_code') +
      _SIGNED_INTEGER.setResultsName('received_bytes') +
      _SIGNED_INTEGER.setResultsName('sent_bytes') +
      _REQUEST +
      _USER_AGENT +
      _WORD.setResultsName('ssl_cipher') +
      _WORD.setResultsName('ssl_protocol') +
      _END_OF_LINE)

  _LINE_STRUCTURES = [
      ('elb_application_accesslog', _APPLICATION_LOG_LINE),
      ('elb_classic_accesslog', _CLASSIC_LOG_LINE),
      ('elb_network_accesslog', _NETWORK_LOG_LINE)]

  VERIFICATION_GRAMMAR = (
      _APPLICATION_LOG_LINE ^ _CLASSIC_LOG_LINE ^ _NETWORK_LOG_LINE)

  def _GetValueFromGroup(self, structure, name, key_name):
    """Retrieves a value from a Pyparsing.Group structure.

    Args:
      structure (pyparsing.ParseResults): tokens from a parsed log line.
      name (str): name of the token.
      key_name (str): key name to retrieve the value of.

    Returns:
      object: value for the specified key or None if not available.
    """
    structure_value = self._GetValueFromStructure(structure, name)
    if structure_value is None:
      return None

    return structure_value.get(key_name)

  def _ParseRecord(self, parser_mediator, key, structure):
    """Parses a pyparsing structure.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      key (str): name of the parsed structure.
      structure (pyparsing.ParseResults): tokens from a parsed log line.

    Raises:
      ParseError: if the structure cannot be parsed.
    """
    destination_list = self._GetValueFromStructure(
        structure, 'destination_list')
    if destination_list:
      destination_list = destination_list.split()

    chosen_cert_serial = self._GetValueFromStructure(
        structure, 'chosen_cert_serial')
    if chosen_cert_serial == '-':
      chosen_cert_serial = None

    classification = self._GetValueFromStructure(structure, 'classification')
    if classification == '-':
      classification = None

    classification_reason = self._GetValueFromStructure(
        structure, 'classification_reason')
    if classification_reason == '-':
      classification_reason = None

    destination_status_code = self._GetValueFromStructure(
        structure, 'destination_status_code')
    if destination_status_code == '-':
      destination_status_code = None

    elb_status_code = self._GetValueFromStructure(structure, 'elb_status_code')
    if elb_status_code == '-':
      elb_status_code = None

    error_reason = self._GetValueFromStructure(structure, 'error_reason')
    if error_reason == '-':
      error_reason = None

    incoming_tls_alert = self._GetValueFromStructure(
        structure, 'incoming_tls_alert')
    if incoming_tls_alert == '-':
      incoming_tls_alert = None

    redirect_url = self._GetValueFromStructure(structure, 'redirect_url')
    if redirect_url == '-':
      redirect_url = None

    ssl_cipher = self._GetValueFromStructure(structure, 'ssl_cipher')
    if ssl_cipher == '-':
      ssl_cipher = None

    ssl_protocol = self._GetValueFromStructure(structure, 'ssl_protocol')
    if ssl_protocol == '-':
      ssl_protocol = None

    tls_named_group = self._GetValueFromStructure(structure, 'tls_named_group')
    if tls_named_group == '-':
      tls_named_group = None

    user_agent = self._GetValueFromStructure(structure, 'user_agent')
    if user_agent == '-':
      user_agent = None

    event_data = AWSELBEventData()
    event_data.request_type = self._GetValueFromStructure(
        structure, 'request_type')
    event_data.resource_identifier = self._GetValueFromStructure(
        structure, 'resource_identifier')
    event_data.source_ip_address = self._GetValueFromGroup(
        structure, 'source_ip_port', 'source_ip_address')
    event_data.source_port = self._GetValueFromGroup(
        structure, 'source_ip_port', 'source_port')
    event_data.destination_ip_address = self._GetValueFromGroup(
        structure, 'destination_ip_port', 'destination_ip_address')
    event_data.destination_port = self._GetValueFromGroup(
        structure, 'destination_ip_port', 'destination_port')
    event_data.request_processing_duration = self._GetValueFromStructure(
        structure, 'request_processing_duration')
    event_data.destination_processing_duration = self._GetValueFromStructure(
        structure, 'destination_processing_duration')
    event_data.response_processing_duration = self._GetValueFromStructure(
        structure, 'response_processing_duration')
    event_data.elb_status_code = elb_status_code
    event_data.destination_status_code = destination_status_code
    event_data.received_bytes = self._GetValueFromStructure(
        structure, 'received_bytes')
    event_data.sent_bytes = self._GetValueFromStructure(structure, 'sent_bytes')
    event_data.request = self._GetValueFromStructure(structure, 'request')
    event_data.user_agent = user_agent
    event_data.ssl_cipher = ssl_cipher
    event_data.ssl_protocol = ssl_protocol
    event_data.destination_group_arn = self._GetValueFromStructure(
        structure, 'destination_group_arn')
    event_data.trace_identifier = self._GetValueFromStructure(
        structure, 'trace_identifier')
    event_data.domain_name = self._GetValueFromStructure(
        structure, 'domain_name')
    event_data.chosen_cert_arn = self._GetValueFromStructure(
        structure, 'chosen_cert_arn')
    event_data.matched_rule_priority = self._GetValueFromStructure(
        structure, 'matched_rule_priority')
    event_data.actions_executed = self._GetValueFromStructure(
        structure, 'actions_executed')
    event_data.redirect_url = redirect_url
    event_data.error_reason = error_reason
    event_data.destination_status_code_list = self._GetValueFromStructure(
        structure, 'destination_status_code_list')
    event_data.classification = classification
    event_data.classification_reason = classification_reason
    event_data.destination_list = destination_list
    event_data.version = self._GetValueFromStructure(structure, 'version')
    event_data.listener = self._GetValueFromStructure(structure, 'listener')
    event_data.connection_duration = self._GetValueFromStructure(
        structure, 'connection_duration')
    event_data.handshake_duration = self._GetValueFromStructure(
        structure, 'handshake_duration')
    event_data.incoming_tls_alert = incoming_tls_alert
    event_data.chosen_cert_serial = chosen_cert_serial
    event_data.tls_named_group = tls_named_group
    event_data.tls_cipher = self._GetValueFromStructure(structure, 'tls_cipher')
    event_data.tls_protocol_version = self._GetValueFromStructure(
        structure, 'tls_protocol_version')
    event_data.alpn_front_end_protocol = self._GetValueFromStructure(
        structure, 'alpn_front_end_protocol')
    event_data.alpn_back_end_protocol = self._GetValueFromStructure(
        structure, 'alpn_back_end_protocol')
    event_data.alpn_client_preference_list = self._GetValueFromStructure(
        structure, 'alpn_client_preference_list')

    response_time_structure = self._GetValueFromStructure(
        structure, 'response_time')
    if response_time_structure:
      event_data.response_time = self._ParseTimeElements(
          response_time_structure)

    request_time_structure = structure.get('request_time')
    if request_time_structure:
      event_data.request_time = self._ParseTimeElements(request_time_structure)

    parser_mediator.ProduceEventData(event_data)

  def _ParseTimeElements(self, time_elements_structure):
    """Parses date and time elements of a log line.

    Args:
      time_elements_structure (pyparsing.ParseResults): date and time elements
          of a log line.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    try:
      # Ensure time_elements_tuple is not a pyparsing.ParseResults otherwise
      # copy.deepcopy() of the dfDateTime object will fail on Python 3.8 with:
      # "TypeError: 'str' object is not callable" due to pyparsing.ParseResults
      # overriding __getattr__ with a function that returns an empty string
      # when named token does not exist.

      if len(time_elements_structure) == 7:
        year, month, day_of_month, hours, minutes, seconds, microseconds = (
            time_elements_structure)

        time_elements_tuple = (
            year, month, day_of_month, hours, minutes, seconds, microseconds)
        date_time = dfdatetime_time_elements.TimeElementsInMicroseconds(
            time_elements_tuple=time_elements_tuple)
      else:
        year, month, day_of_month, hours, minutes, seconds = (
            time_elements_structure)

        time_elements_tuple = (
            year, month, day_of_month, hours, minutes, seconds)
        date_time = dfdatetime_time_elements.TimeElements(
            time_elements_tuple=time_elements_tuple)
        date_time.is_local_time = True

      return date_time

    except (TypeError, ValueError) as exception:
      raise errors.ParseError(
          'Unable to parse time elements with error: {0!s}'.format(exception))

  def CheckRequiredFormat(self, parser_mediator, text_reader):
    """Check if the log record has the minimal structure required by the plugin.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      text_reader (EncodedTextReader): text reader.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    try:
      structure = self._VerifyString(text_reader.lines)
    except errors.ParseError:
      return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'response_time')

    if time_elements_structure:
      try:
        self._ParseTimeElements(time_elements_structure)
      except errors.ParseError:
        return False

    time_elements_structure = self._GetValueFromStructure(
        structure, 'request_time')

    if time_elements_structure:
      try:
        self._ParseTimeElements(time_elements_structure)
      except errors.ParseError:
        return False

    return True


text_parser.TextLogParser.RegisterPlugin(AWSELBTextPlugin)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the MacOS Notification Center plugin."""

import unittest

from plaso.parsers.sqlite_plugins import macos_notification_center

from tests.parsers.sqlite_plugins import test_lib


class MacOSNotificationCenterTest(test_lib.SQLitePluginTestCase):
  """Tests for the MacOS Notification Center plugin."""

  def testProcess(self):
    """Tests the Process function."""
    plugin = macos_notification_center.MacOSNotificationCenterPlugin()
    storage_writer = self._ParseDatabaseFileWithPlugin(
        ['mac_notificationcenter.db'], plugin)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 6)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'body': 'KeePassXC can now be run',
        'bundle_name': 'com.google.santagui',
        'creation_time': '2018-05-02T10:59:18.930155+00:00',
        'data_type': 'macos:notification_center:entry',
        'presented': 1,
        'title': 'Santa'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the YAML-based timeliner configuration file."""

import io
import unittest

from plaso.engine import yaml_timeliner_file
from plaso.lib import errors

from tests import test_lib as shared_test_lib


class YAMLTimelinerConfigurationFileTest(shared_test_lib.BaseTestCase):
  """Tests for the YAML-based timeliner configuration file."""

  # pylint: disable=protected-access

  _FORMATTERS_YAML = {
      'data_type': 'test:fs:stat',
      'attribute_mappings': [{
          'name': 'access_time',
          'description': 'Last Access Time'}],
      'place_holder_event': True}

  def testReadTimelinerDefinition(self):
    """Tests the _ReadTimelinerDefinition function."""
    test_timeliner_file = yaml_timeliner_file.YAMLTimelinerConfigurationFile()

    timeliner_definition = test_timeliner_file._ReadTimelinerDefinition(
        self._FORMATTERS_YAML)

    self.assertIsNotNone(timeliner_definition)
    self.assertEqual(timeliner_definition.data_type, 'test:fs:stat')

    with self.assertRaises(errors.ParseError):
      test_timeliner_file._ReadTimelinerDefinition({})

    with self.assertRaises(errors.ParseError):
      test_timeliner_file._ReadTimelinerDefinition({'bogus': 'error'})

  def testReadFromFileObject(self):
    """Tests the _ReadFromFileObject function."""
    test_file_path = self._GetTestFilePath(['timeliner.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_timeliner_file = yaml_timeliner_file.YAMLTimelinerConfigurationFile()

    with io.open(test_file_path, 'r', encoding='utf-8') as file_object:
      timeliner_definitions = list(test_timeliner_file._ReadFromFileObject(
          file_object))

    self.assertEqual(len(timeliner_definitions), 3)

  def testReadFromFile(self):
    """Tests the ReadFromFile function."""
    test_file_path = self._GetTestFilePath(['timeliner.yaml'])
    self._SkipIfPathNotExists(test_file_path)

    test_timeliner_file = yaml_timeliner_file.YAMLTimelinerConfigurationFile()

    timeliner_definitions = list(test_timeliner_file.ReadFromFile(
        test_file_path))

    self.assertEqual(len(timeliner_definitions), 3)

    self.assertEqual(timeliner_definitions[0].data_type, 'test:fs:stat')


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""Parser for NTFS metadata files."""

import os
import uuid

from dfdatetime import filetime as dfdatetime_filetime

import pyfsntfs

from plaso.containers import events
from plaso.containers import windows_events
from plaso.lib import dtfabric_helper
from plaso.lib import errors
from plaso.lib import specification
from plaso.parsers import interface
from plaso.parsers import manager


class NTFSFileStatEventData(events.EventData):
  """NTFS file system stat event data.

  Attributes:
    access_time (dfdatetime.DateTimeValues): file entry last access date
        and time.
    attribute_type (int): attribute type for example "0x00000030", which
        represents "$FILE_NAME".
    creation_time (dfdatetime.DateTimeValues): file entry creation date
        and time.
    display_name (str): display name.
    entry_modification_time (dfdatetime.DateTimeValues): file entry
         modification date and time.
    file_attribute_flags (int): NTFS file attribute flags.
    file_reference (int): NTFS file reference.
    file_system_type (str): file system type.
    filename (str): name of the file.
    is_allocated (bool): True if the MFT entry is allocated (marked as in use).
    modification_time (dfdatetime.DateTimeValues): file entry last modification
        date and time.
    name (str): name associated with the stat event, for example that of
        a $FILE_NAME attribute or None if not available.
    parent_file_reference (int): NTFS file reference of the parent.
    path_hints (list[str]): hints about the full path of the file.
    symbolic_link_target (str): path of the symbolic link target.
  """

  DATA_TYPE = 'fs:stat:ntfs'

  def __init__(self):
    """Initializes event data."""
    super(NTFSFileStatEventData, self).__init__(data_type=self.DATA_TYPE)
    self.access_time = None
    self.attribute_type = None
    self.creation_time = None
    self.display_name = None
    self.entry_modification_time = None
    self.file_attribute_flags = None
    self.file_reference = None
    self.file_system_type = 'NTFS'
    self.filename = None
    self.is_allocated = None
    self.modification_time = None
    self.name = None
    self.parent_file_reference = None
    self.path_hints = None
    self.symbolic_link_target = None


class NTFSUSNChangeEventData(events.EventData):
  """NTFS USN change event data.

  Attributes:
    file_attribute_flags (int): NTFS file attribute flags.
    filename (str): name of the file associated with the event.
    file_reference (int): NTFS file reference.
    file_system_type (str): file system type.
    parent_file_reference (int): NTFS file reference of the parent.
    offset (int): offset of the USN record relative to the start of the $J data
        stream, from which the event data was extracted.
    update_reason_flags (int): update reason flags.
    update_sequence_number (int): update sequence number.
    update_source_flags (int): update source flags.
    update_time (dfdatetime.DateTimeValues): update date and time.
  """

  DATA_TYPE = 'fs:ntfs:usn_change'

  def __init__(self):
    """Initializes event data."""
    super(NTFSUSNChangeEventData, self).__init__(data_type=self.DATA_TYPE)
    self.file_attribute_flags = None
    self.filename = None
    self.file_reference = None
    self.parent_file_reference = None
    self.offset = None
    self.update_reason_flags = None
    self.update_sequence_number = None
    self.update_source_flags = None
    self.update_time = None


class NTFSMFTParser(interface.FileObjectParser):
  """Parses a NTFS $MFT metadata file."""

  NAME = 'mft'
  DATA_FORMAT = 'NTFS $MFT metadata file'

  _INITIAL_FILE_OFFSET = None

  _MFT_ATTRIBUTE_STANDARD_INFORMATION = 0x00000010
  _MFT_ATTRIBUTE_FILE_NAME = 0x00000030
  _MFT_ATTRIBUTE_OBJECT_ID = 0x00000040
  _MFT_ATTRIBUTE_DATA = 0x00000080

  _NAMESPACE_DOS = 2

  @classmethod
  def GetFormatSpecification(cls):
    """Retrieves the format specification.

    Returns:
      FormatSpecification: format specification.
    """
    format_specification = specification.FormatSpecification(cls.NAME)
    format_specification.AddNewSignature(b'FILE', offset=0)
    return format_specification

  def _GetDateTime(self, filetime):
    """Retrieves the date and time from a FILETIME timestamp.

    Args:
      filetime (int): FILETIME timestamp.

    Returns:
      dfdatetime.DateTimeValues: date and time or None if not set.
    """
    if not filetime:
      return None

    return dfdatetime_filetime.Filetime(timestamp=filetime)

  def _ParseDistributedTrackingIdentifier(
      self, parser_mediator, uuid_string, origin):
    """Extracts data from a Distributed Tracking identifier.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      uuid_string (str): UUID string of the Distributed Tracking identifier.
      origin (str): origin of the event (event source).
    """
    uuid_object = uuid.UUID(uuid_string)

    if uuid_object.version == 1:
      event_data = windows_events.WindowsDistributedLinkTrackingEventData(
          uuid_object, origin)
      parser_mediator.ProduceEventData(event_data)

  def _ParseFileStatAttribute(
      self, parser_mediator, mft_entry, mft_attribute, path_hints):
    """Extract data from a NFTS $STANDARD_INFORMATION or $FILE_NAME attribute.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      mft_entry (pyfsntfs.file_entry): MFT entry.
      mft_attribute (pyfsntfs.attribute): MFT attribute.
      path_hints (list[str]): hints about the full path of the file.
    """
    event_data = NTFSFileStatEventData()
    event_data.attribute_type = mft_attribute.attribute_type
    event_data.display_name = parser_mediator.GetDisplayName()
    event_data.file_reference = mft_entry.file_reference
    event_data.filename = parser_mediator.GetRelativePath()
    event_data.is_allocated = mft_entry.is_allocated()
    event_data.path_hints = path_hints or None
    event_data.symbolic_link_target = mft_entry.get_symbolic_link_target()

    if mft_attribute.attribute_type == self._MFT_ATTRIBUTE_FILE_NAME:
      event_data.file_attribute_flags = mft_attribute.file_attribute_flags
      event_data.name = mft_attribute.name
      event_data.parent_file_reference = mft_attribute.parent_file_reference

    try:
      filetime = mft_attribute.get_access_time_as_integer()
      event_data.access_time = self._GetDateTime(filetime)
    except OverflowError as exception:
      parser_mediator.ProduceExtractionWarning((
          'unable to read the access timestamp from MFT attribute: '
          '0x{0:08x} with error: {1!s}').format(
              exception, mft_attribute.attribute_type))

    try:
      filetime = mft_attribute.get_creation_time_as_integer()
      event_data.creation_time = self._GetDateTime(filetime)
    except OverflowError as exception:
      parser_mediator.ProduceExtractionWarning((
          'unable to read the creation timestamp from MFT attribute: '
          '0x{0:08x} with error: {1!s}').format(
              mft_attribute.attribute_type, exception))

    try:
      filetime = mft_attribute.get_entry_modification_time_as_integer()
      event_data.entry_modification_time = self._GetDateTime(filetime)
    except OverflowError as exception:
      parser_mediator.ProduceExtractionWarning((
          'unable to read the entry modification timestamp from MFT '
          'attribute: 0x{0:08x} with error: {1!s}').format(
              mft_attribute.attribute_type, exception))

    try:
      filetime = mft_attribute.get_modification_time_as_integer()
      event_data.modification_time = self._GetDateTime(filetime)
    except OverflowError as exception:
      parser_mediator.ProduceExtractionWarning((
          'unable to read the modification timestamp from MFT attribute: '
          '0x{0:08x} with error: {1!s}').format(
              mft_attribute.attribute_type, exception))

    parser_mediator.ProduceEventData(event_data)

  def _ParseObjectIDAttribute(
      self, parser_mediator, mft_entry, mft_attribute):
    """Extract data from a NFTS $OBJECT_ID attribute.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      mft_entry (pyfsntfs.file_entry): MFT entry.
      mft_attribute (pyfsntfs.attribute): MFT attribute.
    """
    display_name = '$MFT: {0:d}-{1:d}'.format(
        mft_entry.file_reference & 0xffffffffffff,
        mft_entry.file_reference >> 48)

    if mft_attribute.droid_file_identifier:
      try:
        self._ParseDistributedTrackingIdentifier(
            parser_mediator, mft_attribute.droid_file_identifier,
            display_name)

      except (TypeError, ValueError) as exception:
        parser_mediator.ProduceExtractionWarning((
            'unable to read droid file identifier from attribute: 0x{0:08x} '
            'with error: {1!s}').format(
                mft_attribute.attribute_type, exception))

    if mft_attribute.birth_droid_file_identifier:
      try:
        self._ParseDistributedTrackingIdentifier(
            parser_mediator, mft_attribute.droid_file_identifier,
            display_name)

      except (TypeError, ValueError) as exception:
        parser_mediator.ProduceExtractionWarning((
            'unable to read birth droid file identifier from attribute: '
            '0x{0:08x} with error: {1!s}').format(
                mft_attribute.attribute_type, exception))

  def _ParseMFTEntry(self, parser_mediator, mft_entry):
    """Extracts data from a NFTS $MFT entry.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      mft_entry (pyfsntfs.file_entry): MFT entry.
    """
    data_stream_names = []
    path_hints = []
    standard_information_attribute = None
    standard_information_attribute_index = None

    for attribute_index in range(0, mft_entry.number_of_attributes):
      try:
        mft_attribute = mft_entry.get_attribute(attribute_index)
        if mft_attribute.attribute_type == (
            self._MFT_ATTRIBUTE_STANDARD_INFORMATION):
          standard_information_attribute = mft_attribute
          standard_information_attribute_index = attribute_index

        elif mft_attribute.attribute_type == self._MFT_ATTRIBUTE_FILE_NAME:
          path_hint = mft_entry.get_path_hint(attribute_index)
          self._ParseFileStatAttribute(
              parser_mediator, mft_entry, mft_attribute, [path_hint])
          if mft_attribute.name_space != self._NAMESPACE_DOS:
            path_hints.append(path_hint)

        elif mft_attribute.attribute_type == self._MFT_ATTRIBUTE_OBJECT_ID:
          self._ParseObjectIDAttribute(
              parser_mediator, mft_entry, mft_attribute)

        elif mft_attribute.attribute_type == self._MFT_ATTRIBUTE_DATA:
          data_stream_names.append(mft_attribute.attribute_name)

      except IOError as exception:
        parser_mediator.ProduceExtractionWarning((
            'unable to parse MFT attribute: {0:d} with error: {1!s}').format(
                attribute_index, exception))

    if standard_information_attribute:
      path_hints_with_data_streams = []
      for path_hint in path_hints:
        if not path_hint:
          path_hint = '\\'

        if not data_stream_names:
          path_hints_with_data_streams.append(path_hint)
        else:
          for data_stream_name in data_stream_names:
            if not data_stream_name:
              path_hint_with_data_stream = path_hint
            else:
              path_hint_with_data_stream = '{0:s}:{1:s}'.format(
                  path_hint, data_stream_name)

            path_hints_with_data_streams.append(path_hint_with_data_stream)

      try:
        self._ParseFileStatAttribute(
            parser_mediator, mft_entry, standard_information_attribute,
            path_hints_with_data_streams)
      except IOError as exception:
        parser_mediator.ProduceExtractionWarning((
            'unable to parse MFT attribute: {0:d} with error: {1!s}').format(
                standard_information_attribute_index, exception))

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a NTFS $MFT metadata file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.
    """
    mft_metadata_file = pyfsntfs.mft_metadata_file()

    try:
      mft_metadata_file.open_file_object(file_object)
    except IOError as exception:
      parser_mediator.ProduceExtractionWarning(
          'unable to open $MFT file with error: {0!s}'.format(exception))
      return

    for entry_index in range(0, mft_metadata_file.number_of_file_entries):
      try:
        mft_entry = mft_metadata_file.get_file_entry(entry_index)
        if (not mft_entry.is_empty() and
            mft_entry.base_record_file_reference == 0):
          self._ParseMFTEntry(parser_mediator, mft_entry)

      except IOError as exception:
        parser_mediator.ProduceExtractionWarning((
            'unable to parse MFT entry: {0:d} with error: {1!s}').format(
                entry_index, exception))

    mft_metadata_file.close()


class NTFSUsnJrnlParser(
    interface.FileObjectParser, dtfabric_helper.DtFabricHelper):
  """Parses a NTFS USN change journal."""

  NAME = 'usnjrnl'
  DATA_FORMAT = (
      'NTFS USN change journal ($UsnJrnl:$J) file system metadata file')

  _DEFINITION_FILE = os.path.join(
      os.path.dirname(__file__), 'ntfs.yaml')

  _INITIAL_FILE_OFFSET = None

  # TODO: add support for USN_RECORD_V3 and USN_RECORD_V4 when actually
  # seen to be used.

  def _GetDateTime(self, filetime):
    """Retrieves the date and time from a FILETIME timestamp.

    Args:
      filetime (int): FILETIME timestamp.

    Returns:
      dfdatetime.DateTimeValues: date and time or None if not set.
    """
    if not filetime:
      return None

    return dfdatetime_filetime.Filetime(timestamp=filetime)

  def _ParseUSNChangeJournal(self, parser_mediator, usn_change_journal):
    """Parses an USN change journal.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      usn_change_journal (pyfsntsfs.usn_change_journal): USN change journal.

    Raises:
      ParseError: if an USN change journal record cannot be parsed.
    """
    if not usn_change_journal:
      return

    usn_record_map = self._GetDataTypeMap('usn_record_v2')

    usn_record_data = usn_change_journal.read_usn_record()
    while usn_record_data:
      current_offset = usn_change_journal.get_offset()

      try:
        usn_record = self._ReadStructureFromByteStream(
            usn_record_data, current_offset, usn_record_map)
      except (ValueError, errors.ParseError) as exception:
        raise errors.ParseError((
            'Unable to parse USN record at offset: 0x{0:08x} with error: '
            '{1!s}').format(current_offset, exception))

      # Per MSDN we need to use name offset for forward compatibility.
      name_offset = usn_record.name_offset - 60
      utf16_stream = usn_record.name[name_offset:usn_record.name_size]

      try:
        name_string = utf16_stream.decode('utf-16-le')
      except (UnicodeDecodeError, UnicodeEncodeError) as exception:
        name_string = utf16_stream.decode('utf-16-le', errors='replace')
        parser_mediator.ProduceExtractionWarning((
            'unable to decode USN record name string with error: '
            '{0:s}. Characters that cannot be decoded will be replaced '
            'with "?" or "\\ufffd".').format(exception))

      event_data = NTFSUSNChangeEventData()
      event_data.file_attribute_flags = usn_record.file_attribute_flags
      event_data.file_reference = usn_record.file_reference
      event_data.filename = name_string
      event_data.offset = current_offset
      event_data.parent_file_reference = usn_record.parent_file_reference
      event_data.update_time = self._GetDateTime(usn_record.update_date_time)
      event_data.update_reason_flags = usn_record.update_reason_flags
      event_data.update_sequence_number = usn_record.update_sequence_number
      event_data.update_source_flags = usn_record.update_source_flags

      parser_mediator.ProduceEventData(event_data)

      usn_record_data = usn_change_journal.read_usn_record()

  def ParseFileObject(self, parser_mediator, file_object):
    """Parses a NTFS $UsnJrnl metadata file-like object.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfvfs.
      file_object (dfvfs.FileIO): file-like object.
    """
    fsntfs_volume = pyfsntfs.volume()
    try:
      fsntfs_volume.open_file_object(file_object)
    except IOError as exception:
      parser_mediator.ProduceExtractionWarning(
          'unable to open NTFS volume with error: {0!s}'.format(exception))
      return

    try:
      usn_change_journal = fsntfs_volume.get_usn_change_journal()
      self._ParseUSNChangeJournal(parser_mediator, usn_change_journal)
    finally:
      fsntfs_volume.close()


manager.ParsersManager.RegisterParsers([NTFSMFTParser, NTFSUsnJrnlParser])



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the SAM Users Account information plugin."""

import unittest

from plaso.parsers.winreg_plugins import sam_users

from tests.parsers.winreg_plugins import test_lib


class SAMUsersWindowsRegistryPluginTest(test_lib.RegistryPluginTestCase):
  """Tests the SAM Users Account information plugin."""

  def testFilters(self):
    """Tests the FILTERS class attribute."""
    plugin = sam_users.SAMUsersWindowsRegistryPlugin()

    key_path = 'HKEY_LOCAL_MACHINE\\SAM\\SAM\\Domains\\Account\\Users'
    self._AssertFiltersOnKeyPath(plugin, key_path)

    self._AssertNotFiltersOnKeyPath(plugin, 'HKEY_LOCAL_MACHINE\\Bogus')

  def testProcess(self):
    """Tests the Process function."""
    test_file_entry = self._GetTestFileEntry(['SAM'])
    key_path = 'HKEY_LOCAL_MACHINE\\SAM\\SAM\\Domains\\Account\\Users'

    win_registry = self._GetWinRegistryFromFileEntry(test_file_entry)
    registry_key = win_registry.GetKeyByPath(key_path)

    plugin = sam_users.SAMUsersWindowsRegistryPlugin()
    storage_writer = self._ParseKeyWithPlugin(
        registry_key, plugin, file_entry=test_file_entry)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    expected_event_values = {
        'account_rid': 500,
        'comments': 'Built-in account for administering the computer/domain',
        'data_type': 'windows:registry:sam_users',
        'last_login_time': '2010-11-20T21:48:12.5692440+00:00',
        'last_password_set_time': '2010-11-20T21:56:34.7436870+00:00',
        'last_written_time': '2014-09-24T03:36:06.3588374+00:00',
        'login_count': 6,
        'username': 'Administrator'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 0)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the Google Chrome history custom event formatter helpers."""

import unittest

from plaso.formatters import chrome

from tests.formatters import test_lib


class ChromeHistoryTypedCountFormatterHelperTest(
    test_lib.EventFormatterTestCase):
  """Tests for the Google Chrome history typed count formatter helper."""

  def testFormatEventValues(self):
    """Tests the FormatEventValues function."""
    formatter_helper = chrome.ChromeHistoryTypedCountFormatterHelper()

    output_mediator = self._CreateOutputMediator()

    event_values = {'typed_count': 0}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(
        event_values['url_typed_string'], '(URL not typed directly)')

    event_values = {'typed_count': 1}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(
        event_values['url_typed_string'], '(URL typed 1 time)')

    event_values = {'typed_count': 3}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(
        event_values['url_typed_string'], '(URL typed 3 times)')

    event_values = {'typed_count': -1}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertEqual(event_values['url_typed_string'], -1)

    event_values = {'typed_count': None}
    formatter_helper.FormatEventValues(output_mediator, event_values)
    self.assertNotIn('url_typed_string', event_values)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests for the output and formatting multi-processing engine."""

import io
import os
import unittest

from plaso.engine import configurations
from plaso.lib import definitions
from plaso.multi_process import output_engine
from plaso.output import dynamic
from plaso.output import interface as output_interface
from plaso.output import mediator as output_mediator
from plaso.storage import factory as storage_factory

from tests import test_lib as shared_test_lib
from tests.containers import test_lib as containers_test_lib
from tests.multi_process import test_lib


class TestOutputModule(output_interface.OutputModule):
  """Output module for testing.

  Attributes:
    events (list[tuple[EventObject, EventData]]): event written to the output.
    macb_groups (list[list[tuple[EventObject, EventData]]]): MACB groups of
        events written to the output.
  """

  # pylint: disable=arguments-renamed,unused-argument

  NAME = 'psort_test'

  def __init__(self):
    """Initializes an output module."""
    super(TestOutputModule, self).__init__()
    self.events = []
    self.macb_groups = []

  def _GetFieldValues(
      self, output_mediator_object, event, event_data, event_data_stream,
      event_tag):
    """Retrieves the output field values.

    Args:
      output_mediator_object (OutputMediator): mediates interactions between
          output modules and other components, such as storage and dfVFS.
      event (EventObject): event.
      event_data (EventData): event data.
      event_data_stream (EventDataStream): event data stream.
      event_tag (EventTag): event tag.

    Returns:
      dict[str, str]: output field values per name.
    """
    return {}

  def _WriteFieldValues(self, output_mediator_object, field_values):
    """Writes field values to the output.

    Args:
      output_mediator_object (OutputMediator): mediates interactions between
          output modules and other components, such as storage and dfVFS.
      field_values (dict[str, str]): output field values per name.
    """
    self.events.append(field_values)

  def WriteFieldValuesOfMACBGroup(self, output_mediator_object, macb_group):
    """Writes field values of a MACB group to the output.

    Args:
      output_mediator_object (OutputMediator): mediates interactions between
          output modules and other components, such as storage and dfVFS.
      macb_group (list[dict[str, str]]): group of output field values per name
          with identical timestamps, attributes and values.
    """
    self.events.extend(macb_group)
    self.macb_groups.append(macb_group)

  def WriteHeader(self, output_mediator_object):
    """Writes the header to the output.

    Args:
      output_mediator_object (OutputMediator): mediates interactions between
          output modules and other components, such as storage and dfVFS.
    """
    return


class PsortEventHeapTest(test_lib.MultiProcessingTestCase):
  """Tests for the psort events heap."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'timestamp': 5134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_METADATA_MODIFICATION},
      {'_parser_chain': 'test_parser',
       'data_type': 'test:event',
       'display_name': '/dev/none',
       'filename': '/dev/none',
       'text': 'text',
       'timestamp': 2345871286,
       'timestamp_desc': definitions.TIME_DESCRIPTION_METADATA_MODIFICATION,
       'var': 'Issue: False, Closed: True'}]

  def testNumberOfEvents(self):
    """Tests the number_of_events property."""
    event_heap = output_engine.PsortEventHeap()
    self.assertEqual(event_heap.number_of_events, 0)

  def testPopEvent(self):
    """Tests the PopEvent function."""
    event_heap = output_engine.PsortEventHeap()

    self.assertEqual(len(event_heap._heap), 0)

    test_event = event_heap.PopEvent()
    self.assertIsNone(test_event)

    for event, event_data, event_data_stream in (
        containers_test_lib.CreateEventsFromValues(self._TEST_EVENTS)):
      event_heap.PushEvent(event, event_data, event_data_stream)

    self.assertEqual(len(event_heap._heap), 2)

    test_event = event_heap.PopEvent()
    self.assertIsNotNone(test_event)

    self.assertEqual(len(event_heap._heap), 1)

  def testPopEvents(self):
    """Tests the PopEvents function."""
    event_heap = output_engine.PsortEventHeap()

    self.assertEqual(len(event_heap._heap), 0)

    test_events = list(event_heap.PopEvents())
    self.assertEqual(len(test_events), 0)

    for event, event_data, event_data_stream in (
        containers_test_lib.CreateEventsFromValues(self._TEST_EVENTS)):
      event_heap.PushEvent(event, event_data, event_data_stream)

    self.assertEqual(len(event_heap._heap), 2)

    test_events = list(event_heap.PopEvents())
    self.assertEqual(len(test_events), 2)

    self.assertEqual(len(event_heap._heap), 0)

  def testPushEvent(self):
    """Tests the PushEvent function."""
    event_heap = output_engine.PsortEventHeap()

    self.assertEqual(len(event_heap._heap), 0)

    event, event_data, event_data_stream = (
        containers_test_lib.CreateEventFromValues(self._TEST_EVENTS[0]))
    event_heap.PushEvent(event, event_data, event_data_stream)

    self.assertEqual(len(event_heap._heap), 1)


class OutputAndFormattingMultiProcessEngineTest(
    test_lib.MultiProcessingTestCase):
  """Tests for the multi-processing engine."""

  # pylint: disable=protected-access

  _TEST_EVENTS = [
      {'data_type': 'test:event',
       'timestamp': 5134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_WRITTEN},
      {'data_type': 'test:event',
       'timestamp': 5134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_LAST_ACCESS},
      {'data_type': 'test:event',
       'timestamp': 2134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'test:event',
       'timestamp': 9134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'test:event',
       'timestamp': 5134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_METADATA_MODIFICATION},
      {'data_type': 'test:event',
       'timestamp': 5134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_CREATION},
      {'data_type': 'test:event',
       'timestamp': 15134324321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'test:event',
       'timestamp': 5134324322,
       'timestamp_desc': definitions.TIME_DESCRIPTION_CREATION},
      {'data_type': 'test:event',
       'timestamp': 5134324322,
       'timestamp_desc': definitions.TIME_DESCRIPTION_LAST_ACCESS},
      {'data_type': 'test:event',
       'timestamp': 5134324322,
       'timestamp_desc': definitions.TIME_DESCRIPTION_METADATA_MODIFICATION},
      {'data_type': 'test:event',
       'timestamp': 5134324322,
       'timestamp_desc': definitions.TIME_DESCRIPTION_WRITTEN},
      {'data_type': 'test:event',
       'timestamp': 5134324322,
       'timestamp_desc': definitions.TIME_DESCRIPTION_WRITTEN},
      {'data_type': 'test:event',
       'timestamp': 5134324322,
       'text': 'Another text',
       'timestamp_desc': definitions.TIME_DESCRIPTION_LAST_ACCESS},
      {'data_type': 'test:event',
       'timestamp': 5134324322,
       'text': 'Another text',
       'timestamp_desc': definitions.TIME_DESCRIPTION_METADATA_MODIFICATION},
      {'data_type': 'test:event',
       'text': 'Another text',
       'timestamp': 5134324322,
       'timestamp_desc': definitions.TIME_DESCRIPTION_WRITTEN},
      {'data_type': 'test:event',
       'timestamp': 5134024321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN},
      {'data_type': 'test:event',
       'timestamp': 5134024321,
       'timestamp_desc': definitions.TIME_DESCRIPTION_UNKNOWN}]

  def _CreateTestStorageFile(self, path):
    """Creates a storage file for testing.

    Args:
      path (str): path.
    """
    storage_file = storage_factory.StorageFactory.CreateStorageFile(
        definitions.DEFAULT_STORAGE_FORMAT)
    storage_file.Open(path=path, read_only=False)

    # TODO: add preprocessing information.

    for event, event_data, event_data_stream in (
        containers_test_lib.CreateEventsFromValues(self._TEST_EVENTS)):
      storage_file.AddAttributeContainer(event_data_stream)

      event_data.SetEventDataStreamIdentifier(event_data_stream.GetIdentifier())
      storage_file.AddAttributeContainer(event_data)

      event.SetEventDataIdentifier(event_data.GetIdentifier())
      storage_file.AddAttributeContainer(event)

    storage_file.Close()

  # TODO: add test for _ExportEvent.

  def testInternalExportEvents(self):
    """Tests the _ExportEvents function."""
    formatters_directory_path = self._GetDataFilePath(['formatters'])

    output_module = TestOutputModule()

    test_engine = output_engine.OutputAndFormattingMultiProcessEngine()

    with shared_test_lib.TempDirectory() as temp_directory:
      temp_file = os.path.join(temp_directory, 'storage.plaso')
      self._CreateTestStorageFile(temp_file)

      storage_reader = (
          storage_factory.StorageFactory.CreateStorageReaderForFile(temp_file))

      output_mediator_object = output_mediator.OutputMediator(
          storage_reader, data_location=shared_test_lib.TEST_DATA_PATH)
      output_mediator_object.ReadMessageFormattersFromDirectory(
          formatters_directory_path)

      test_engine._ExportEvents(
          storage_reader, output_module, deduplicate_events=False)

    self.assertEqual(len(output_module.events), 17)
    self.assertEqual(len(output_module.macb_groups), 3)

  def testInternalExportEventsDeduplicate(self):
    """Tests the _ExportEvents function with deduplication."""
    formatters_directory_path = self._GetDataFilePath(['formatters'])

    output_module = TestOutputModule()

    test_engine = output_engine.OutputAndFormattingMultiProcessEngine()

    with shared_test_lib.TempDirectory() as temp_directory:
      temp_file = os.path.join(temp_directory, 'storage.plaso')
      self._CreateTestStorageFile(temp_file)

      storage_reader = (
          storage_factory.StorageFactory.CreateStorageReaderForFile(temp_file))

      output_mediator_object = output_mediator.OutputMediator(
          storage_reader, data_location=shared_test_lib.TEST_DATA_PATH)

      output_mediator_object.ReadMessageFormattersFromDirectory(
          formatters_directory_path)

      test_engine._ExportEvents(storage_reader, output_module)

    self.assertEqual(len(output_module.events), 15)
    self.assertEqual(len(output_module.macb_groups), 3)

  # TODO: add test for _FlushExportBuffer.

  def testExportEvents(self):
    """Tests the ExportEvents function."""
    test_file_path = self._GetTestFilePath(['psort_test.plaso'])
    self._SkipIfPathNotExists(test_file_path)

    test_file_object = io.StringIO()

    storage_reader = storage_factory.StorageFactory.CreateStorageReaderForFile(
        test_file_path)

    output_module = dynamic.DynamicOutputModule()
    output_module._file_object = test_file_object

    configuration = configurations.ProcessingConfiguration()
    configuration.data_location = shared_test_lib.DATA_PATH
    configuration.preferred_language = 'en-US'

    test_engine = output_engine.OutputAndFormattingMultiProcessEngine()

    test_engine.ExportEvents(storage_reader, output_module, configuration)

    output = test_file_object.getvalue()
    lines = output.split('\n')

    self.assertEqual(len(lines), 22)

    expected_line = (
        '2014-11-18T01:15:43.000000+00:00,'
        'Content Modification Time,'
        'LOG,'
        'Log File,'
        '[---] last message repeated 5 times ---,'
        'text/syslog_traditional,'
        'OS:/tmp/test/test_data/syslog,'
        'repeated')
    self.assertEqual(lines[14], expected_line)


if __name__ == '__main__':
  unittest.main()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Tests the for bodyfile parser."""

import unittest

from plaso.parsers import bodyfile

from tests.parsers import test_lib


class BodyfileTest(test_lib.ParserTestCase):
  """Tests the for bodyfile parser."""

  def testParse(self):
    """Tests the Parse function."""
    parser = bodyfile.BodyfileParser()
    storage_writer = self._ParseFile(['bodyfile'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 24)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 0)

    # Test this entry:
    # 0|/a_directory/another_file|16|r/rrw-------|151107|5000|22|1337961583|
    # 1337961584|1337961585|0

    expected_event_values = {
        'access_time': '2012-05-25T15:59:43',
        'change_time': '2012-05-25T15:59:45',
        'creation_time': None,
        'data_type': 'fs:bodyfile:entry',
        'filename': '/a_directory/another_file',
        'group_identifier': 5000,
        'inode': 16,
        'modification_time': '2012-05-25T15:59:44',
        'owner_identifier': '151107'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)

  def testParseOnCorruptFile(self):
    """Tests the Parse function on a corrupt bodyfile."""
    parser = bodyfile.BodyfileParser()
    storage_writer = self._ParseFile(['bodyfile.corrupt'], parser)

    number_of_event_data = storage_writer.GetNumberOfAttributeContainers(
        'event_data')
    self.assertEqual(number_of_event_data, 3)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'extraction_warning')
    self.assertEqual(number_of_warnings, 0)

    number_of_warnings = storage_writer.GetNumberOfAttributeContainers(
        'recovery_warning')
    self.assertEqual(number_of_warnings, 1)

    # Event data extracted from line with unescaped \r character.
    expected_event_values = {
        'access_time': '2012-05-25T16:00:53',
        'change_time': '2012-05-25T16:01:03',
        'creation_time': None,
        'data_type': 'fs:bodyfile:entry',
        'filename': '/passwords\r.txt',
        'inode': 15,
        'modification_time': '2012-05-25T16:00:53'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 1)
    self.CheckEventData(event_data, expected_event_values)

    # Event data extracted from line with unescaped \\ character.
    expected_event_values = {
        'access_time': '2021-01-12T12:26:01',
        'change_time': '2019-03-19T04:37:22',
        'creation_time': '2019-03-19T04:37:22',
        'data_type': 'fs:bodyfile:entry',
        'filename': '/Windows\\System32',
        'inode': 75520,
        'modification_time': '2021-01-12T12:26:01'}

    event_data = storage_writer.GetAttributeContainerByIndex('event_data', 2)
    self.CheckEventData(event_data, expected_event_values)


if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
"""SQLite parser plugin for Windows Diagnosis EventTranscript database file."""

import json

from dfdatetime import filetime as dfdatetime_filetime

from plaso.containers import events
from plaso.parsers import sqlite
from plaso.parsers.sqlite_plugins import interface


class WindowsEventTranscriptEventData(events.EventData):
  """Windows diagnosis EventTranscript event data.

  Attributes:
    application_name (str): Application name.
    application_root_directory (str): Application root directory.
    application_version (str): Application version.
    compressed_payload_size (int): Size of the compressed payload.
    event_keywords (int): Event keywords
    event_name_hash (int): Hash of full event name.
    event_name (str): Diagnosis full event name.
    friendly_logging_binary_name (str): Friendly name for logging binary.
    ikey (str): iKey
    is_core (int): Boolean value represented as an integer.
    logging_binary_name (str): Binary that generated the event.
    name (str): Name of the payload, similar to event name.
    producer_identifier (int): Identifier of the EventTranscript event producer.
        provider group.
    provider_group_identifier (int): Identifier of the EventTranscript event
    recorded_time (dfdatetime.DateTimeValues): date and time the entry
        was recorded.
    user_identifier (str): Windows Security identifier (SID) of a user account.
    version (str): Payload version
  """

  DATA_TYPE = 'windows:diagnosis:eventtranscript'

  def __init__(self):
    """Initializes event data."""
    super(WindowsEventTranscriptEventData, self).__init__(
        data_type=self.DATA_TYPE)
    self.application_name = None
    self.application_root_directory = None
    self.application_version = None
    self.compressed_payload_size = None
    self.event_keywords = None
    self.event_name_hash = None
    self.event_name = None
    self.friendly_logging_binary_name = None
    self.ikey = None
    self.is_core = None
    self.logging_binary_name = None
    self.name = None
    self.producer_identifier = None
    self.provider_group_identifier = None
    self.recorded_time = None
    self.user_identifier = None
    self.version = None


class EventTranscriptPlugin(interface.SQLitePlugin):
  """SQLite parser plugin for Windows diagnosis EventTranscript database files.

  The Windows diagnosis EventTranscript database file is typically stored in:
  EventTranscript.db
  """

  NAME = 'windows_eventtranscript'
  DATA_FORMAT = (
      'Windows diagnosis EventTranscript SQLite database (EventTranscript.db) '
      'file')

  REQUIRED_STRUCTURE = {
      'events_persisted': frozenset([
          'sid',
          'timestamp',
          'payload',
          'full_event_name',
          'full_event_name_hash',
          'event_keywords',
          'is_core',
          'provider_group_id',
          'logging_binary_name',
          'friendly_logging_binary_name',
          'compressed_payload_size',
          'producer_id'])}

  QUERIES = [
      (('SELECT events_persisted.sid,'
        'events_persisted.timestamp,'
        'events_persisted.payload,'
        'events_persisted.full_event_name,'
        'events_persisted.full_event_name_hash,'
        'events_persisted.event_keywords,'
        'events_persisted.is_core,'
        'events_persisted.provider_group_id,'
        'events_persisted.logging_binary_name,'
        'events_persisted.friendly_logging_binary_name,'
        'events_persisted.compressed_payload_size,'
        'events_persisted.producer_id '
        'from events_persisted'),
        'ParseEventTranscriptRow')]

  SCHEMAS = [{
      'events_persisted': (
          'CREATE TABLE events_persisted ('
          'sid TEXT,'
          'timestamp INTEGER,'
          'payload TEXT,'
          'full_event_name TEXT,'
          'full_event_name_hash INTEGER,'
          'event_keywords INTEGER,'
          'is_core INTEGER, '
          'provider_group_id INTEGER,'
          'logging_binary_name TEXT,'
          'friendly_logging_binary_name TEXT, '
          'compressed_payload_size INTEGER,'
          'producer_id INTEGER,'
          'extra1 TEXT, '
          'extra2 TEXT,'
          'extra3 TEXT,'
          'FOREIGN KEY(provider_group_id) '
          'REFERENCES provider_groups(group_id), '
          'CONSTRAINT fk_producer_id '
          'FOREIGN KEY(producer_id) '
          'REFERENCES producers(producer_id) ON DELETE CASCADE)')}]

  def _GetDateTimeRowValue(self, query_hash, row, value_name):
    """Retrieves a date and time value from the row.

    Args:
      query_hash (int): hash of the query, that uniquely identifies the query
          that produced the row.
      row (sqlite3.Row): row.
      value_name (str): name of the value.

    Returns:
      dfdatetime.Filetime: date and time value or None if not available.
    """
    timestamp = self._GetRowValue(query_hash, row, value_name)
    if timestamp is None:
      return None

    return dfdatetime_filetime.Filetime(timestamp=timestamp)

  def ParseEventTranscriptRow(
      self, parser_mediator, query, row, **unused_kwargs):
    """Parses EventTranscript row.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      query (str): query that created the row.
      row (sqlite3.Row): row.
    """
    query_hash = hash(query)

    event_data = WindowsEventTranscriptEventData()
    event_data.compressed_payload_size = self._GetRowValue(
        query_hash, row, 'compressed_payload_size')
    event_data.event_keywords = self._GetRowValue(
        query_hash, row, 'event_keywords')
    event_data.event_name = self._GetRowValue(
        query_hash, row, 'full_event_name')
    event_data.event_name_hash = self._GetRowValue(
        query_hash, row, 'full_event_name_hash')
    event_data.friendly_logging_binary_name = self._GetRowValue(
        query_hash, row, 'friendly_logging_binary_name')
    event_data.is_core = self._GetRowValue(
        query_hash, row, 'is_core')
    event_data.logging_binary_name = self._GetRowValue(
        query_hash, row, 'logging_binary_name')
    event_data.producer_identifier = self._GetRowValue(
        query_hash, row, 'producer_id')
    event_data.provider_group_identifier = self._GetRowValue(
        query_hash, row, 'provider_group_id')
    event_data.recorded_time = self._GetDateTimeRowValue(
        query_hash, row, 'timestamp')
    # If the user identifier is an empty string set it to None.
    event_data.user_identifier = self._GetRowValue(
        query_hash, row, 'sid') or None

    # Parse the payload.
    payload = json.loads(self._GetRowValue(query_hash, row, 'payload'))
    payload_data = payload['data']
    payload_name = payload['name']

    event_data.ikey = payload['iKey']
    event_data.name = payload_name
    event_data.version = payload['ver']

    # TODO: add support for
    # data = json.dumps(payload_data, separators=(',',':'))
    # extension = json.dumps(payload['ext'], separators=(',',':'))

    # Microsoft.Windows.Inventory.Core.InventoryApplicationAdd
    if payload_name == (
        'Microsoft.Windows.Inventory.Core.InventoryApplicationAdd'):
      event_data.application_name = payload_data['Name']
      event_data.application_version = payload_data['Version']
      event_data.application_root_directory = payload_data['RootDirPath']

      # TODO: generate event for: payload_data['InstallDate']

    # Win32kTraceLogging.AppInteractivitySummary
    elif payload_name == 'Win32kTraceLogging.AppInteractivitySummary':
      event_data.application_version = payload_data['AppVersion']

    # TODO: generate event for: payload['time']

    parser_mediator.ProduceEventData(event_data)


sqlite.SQLiteParser.RegisterPlugin(EventTranscriptPlugin)



# -*- coding: utf-8 -*-
"""Parser for Windows Program Compatibility Assistant (PCA) log files."""

from dfdatetime import time_elements as dfdatetime_time_elements

from plaso.containers import events
from plaso.lib import errors
from plaso.parsers import dsv_parser
from plaso.parsers import manager


class WindowsPCAEventData(events.EventData):
  """Windows PCA (Program Compatibility Assistant) event data.

  Attributes:
    description (str): description of the executable.
    executable (str): executable filename.
    exit_code (str): final result of the execution.
    last_execution_time (dfdatetime.DateTimeValues): entry last execution
        date and time.
    program_identifier (str): program identifier.
    run_status (str): execution status.
    vendor (str): vendor of executed software.
    version (str): version of executed software.
  """

  DATA_TYPE = 'windows:pca_log:entry'

  def __init__(self):
    """Initializes event data."""
    super(WindowsPCAEventData, self).__init__(data_type=self.DATA_TYPE)
    self.description = None
    self.executable = None
    self.exit_code = None
    self.last_execution_time = None
    self.program_identifier = None
    self.run_status = None
    self.vendor = None
    self.version = None


class WindowsPCABaseParser(dsv_parser.DSVParser):
  """Shared code for parsing Program Compatibility Assistant (PCA) log files."""

  # pylint: disable=abstract-method

  DELIMITER = '|'

  COLUMNS = ()

  _MINIMUM_NUMBER_OF_COLUMNS = None

  def _ParseDateTime(self, date_time_string):
    """Parses date and time elements of a log line.

    Args:
      date_time_string (string): date and time string.

    Returns:
      dfdatetime.TimeElements: date and time value.

    Raises:
      ParseError: if a valid date and time value cannot be derived from
          the time elements.
    """
    date_time = dfdatetime_time_elements.TimeElementsInMilliseconds()

    try:
      date_time.CopyFromDateTimeString(date_time_string)
    except ValueError:
      raise errors.ParseError('Unsupported date and time string: {0!s}'.format(
          date_time_string))

    return date_time

  def VerifyRow(self, parser_mediator, row):
    """Verifies if a line of the file is in the expected format.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.

    Returns:
      bool: True if this is the correct parser, False otherwise.
    """
    if len(row) < self._MINIMUM_NUMBER_OF_COLUMNS:
      return False

    try:
      self._ParseDateTime(row['datetime'])
    except errors.ParseError:
      return False

    return '\\' in row['program']


class WindowsPCADB0Parser(WindowsPCABaseParser):
  """Parses Windows Program Compatibility Assistant DB0 log files."""

  NAME = 'winpca_db0'
  DATA_FORMAT = 'Windows PCA DB0 log file'

  COLUMNS = [
      'datetime', 'run_status', 'program', 'description', 'vendor',
      'version', 'program_id', 'exit_code']

  _MINIMUM_NUMBER_OF_COLUMNS = 8

  def ParseRow(self, parser_mediator, row_offset, row):  # pylint: disable=unused-argument
    """Parses a line of the log file and produces events.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      row_offset (int): offset of the line from which the row was extracted.
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.
    """
    try:
      last_execution_time = self._ParseDateTime(row['datetime'])
    except errors.ParseError:
      parser_mediator.ProduceExtractionWarning(
          'Unsupported date and time string: {0!s}'.format(row['datetime']))

    event_data = WindowsPCAEventData()
    event_data.description = row['description']
    event_data.executable = row['program']
    event_data.exit_code = row['exit_code']
    event_data.last_execution_time = last_execution_time
    event_data.program_identifier = row['program_id']
    event_data.run_status = row['run_status']
    event_data.vendor = row['vendor']
    event_data.version = row['version']

    parser_mediator.ProduceEventData(event_data)


class WindowsPCADicParser(WindowsPCABaseParser):
  """Parses the Windows Program Compatibility Assistant DIC log files."""

  NAME = 'winpca_dic'
  DATA_FORMAT = 'Windows PCA DIC log file'

  COLUMNS = ['program', 'datetime']

  _MINIMUM_NUMBER_OF_COLUMNS = 2

  def ParseRow(self, parser_mediator, row_offset, row):
    """Parses a line of the log file and produces events.

    Args:
      parser_mediator (ParserMediator): mediates interactions between parsers
          and other components, such as storage and dfVFS.
      row_offset (int): offset of the line from which the row was extracted.
      row (dict[str, str]): fields of a single row, as specified in COLUMNS.
    """
    try:
      last_execution_time = self._ParseDateTime(row['datetime'])
    except errors.ParseError:
      parser_mediator.ProduceExtractionWarning(
          'Unsupported date and time string: {0!s}'.format(row['datetime']))

    event_data = WindowsPCAEventData()
    event_data.executable = row['program']
    event_data.last_execution_time = last_execution_time

    parser_mediator.ProduceEventData(event_data)


manager.ParsersManager.RegisterParsers([
    WindowsPCADicParser, WindowsPCADB0Parser])
